@@ -19,6 +19,7 @@ module IDE.Command (
     mkActions
 ,   menuDescription
 ,   makeMenu
+,   canQuit
 ,   quit
 ,   aboutDialog
 ,   buildStatusbar
@@ -26,7 +27,7 @@ module IDE.Command (
 ,   setSensitivity
 ,   updateRecentEntries
 ,   handleSpecialKeystrokes
-,   registerEvents
+,   registerLeksahEvents
 ,   instrumentWindow
 ,   instrumentSecWindow
 ) where
@@ -41,7 +42,7 @@ import Graphics.UI.Gtk
         aboutDialogSetLicense, aboutDialogSetComments,
         aboutDialogSetCopyright, aboutDialogSetVersion, aboutDialogSetName,
         aboutDialogNew, mainQuit, widgetHide, widgetShow,
-        widgetSetSensitive, castToWidget, separatorMenuItemNew,
+        castToWidget, separatorMenuItemNew,
         containerGetChildren, Menu, widgetSetSizeRequest, toolbarSetStyle,
         toolbarSetIconSize, castToToolbar, castToMenuBar,
         uiManagerGetWidget, uiManagerGetAccelGroup, onActionActivate,
@@ -71,16 +72,16 @@ import IDE.Pane.Log
 import IDE.Session
 import IDE.Pane.Modules
 import IDE.Find
+import IDE.Pane.Info
 import IDE.Utils.FileUtils
 import IDE.Utils.GUIUtils
-import IDE.Pane.References
 import Paths_leksah
 import IDE.GUIHistory
 import IDE.Metainfo.Provider
        (getWorkspaceInfo, getIdentifierDescr, updateWorkspaceInfo,
         updateSystemInfo, rebuildSystemInfo, rebuildWorkspaceInfo)
 import IDE.NotebookFlipper
-import IDE.ImportTool (addAllImports)
+import IDE.ImportTool (resolveErrors)
 import IDE.LogRef
 import IDE.Debug
 import System.Directory (doesFileExist)
@@ -108,16 +109,18 @@ import IDE.Pane.Grep (getGrep)
 mkActions :: [ActionDescr IDERef]
 mkActions =
     [AD "File" "_File" Nothing Nothing (return ()) [] False
-    ,AD "FileNew" "_New" Nothing (Just "gtk-new")
+    ,AD "FileNew" "_New Module..." Nothing (Just "gtk-new")
+        (packageTry_ $ addModule []) [] False
+    ,AD "FileNewTextFile" "_New Text File" Nothing Nothing
         fileNew [] False
-    ,AD "FileOpen" "_Open" Nothing (Just "gtk-open")
+    ,AD "FileOpen" "_Open..." Nothing (Just "gtk-open")
         fileOpen [] False
-    ,AD "RecentFiles" "_Recent Files" Nothing Nothing (return ()) [] False
+    ,AD "RecentFiles" "Open _Recent" Nothing Nothing (return ()) [] False
     ,AD "FileRevert" "_Revert" Nothing Nothing
         fileRevert [] False
     ,AD "FileSave" "_Save" Nothing (Just "gtk-save")
         (do fileSave False; return ()) [] False
-    ,AD "FileSaveAs" "Save _As" Nothing (Just "gtk-save-as")
+    ,AD "FileSaveAs" "Save _As..." Nothing (Just "gtk-save-as")
         (do fileSave True; return ()) [] False
     ,AD "FileSaveAll" "Save A_ll" Nothing Nothing
         (do fileSaveAll (\ b -> return (bufferName b /= "_Eval.hs")); return ()) [] False
@@ -176,65 +179,65 @@ mkActions =
         (align ':') [] False
 
     ,AD "Workspace" "_Workspace" Nothing Nothing (return ()) [] False
-    ,AD "NewWorkspace" "_New Workspace" Nothing Nothing
-        workspaceNew [] False
-    ,AD "OpenWorkspace" "_Open Workspace" Nothing Nothing
-        workspaceOpen [] False
-    ,AD "RecentWorkspaces" "_Recent Workspaces" Nothing Nothing (return ()) [] False
-    ,AD "CloseWorkspace" "_Close Workspace" Nothing Nothing
+    ,AD "NewWorkspace" "_New..." Nothing Nothing
+        (workspaceNew >> showWorkspace) [] False
+    ,AD "OpenWorkspace" "_Open..." Nothing Nothing
+        (workspaceOpen >> showWorkspace) [] False
+    ,AD "RecentWorkspaces" "Open _Recent" Nothing Nothing (return ()) [] False
+    ,AD "CloseWorkspace" "_Close" Nothing Nothing
         workspaceClose [] False
 
-    ,AD "CleanWorkspace" "Cl_ean Workspace" (Just "Cleans all packages") (Just "ide_clean")
-        workspaceClean [] False
-    ,AD "MakeWorkspace" "_Make Workspace" (Just "Makes all of this workspace") (Just "ide_configure")
-        workspaceMake [] False
+    ,AD "CleanWorkspace" "Cl_ean" (Just "Cleans all packages") (Just "ide_clean")
+        (workspaceTry_ workspaceClean) [] False
+    ,AD "MakeWorkspace" "_Make" (Just "Makes all of this workspace") (Just "ide_configure")
+        (workspaceTry_ workspaceMake) [] False
     ,AD "NextError" "_Next Error" (Just "Go to the next error") (Just "ide_error_next")
         nextError [] False
     ,AD "PreviousError" "_Previous Error" (Just "Go to the previous error") (Just "ide_error_prev")
         previousError [] False
 
     ,AD "Package" "_Package" Nothing Nothing (return ()) [] False
-    ,AD "NewPackage" "_New Package" Nothing Nothing
-        workspacePackageNew [] False
+    ,AD "NewPackage" "_New..." Nothing Nothing
+        (showWorkspace >> workspaceTry_ workspacePackageNew) [] False
+    ,AD "AddPackage" "_Add..." Nothing Nothing
+        (showWorkspace >> workspaceTry_ workspaceAddPackage) [] False
 --    ,AD "RecentPackages" "_Recent Packages" Nothing Nothing (return ()) [] False
-    ,AD "EditPackage" "_Edit Package" Nothing Nothing
-        packageEdit [] False
+    ,AD "EditPackage" "_Edit" Nothing Nothing
+        (packageTry_ packageEdit) [] False
 --    ,AD "RemovePackage" "_Close Package" Nothing Nothing
 --        removePackage [] False
 
     ,AD "PackageFlags" "Edit Flags" (Just "Edit the package flags") Nothing
         (getFlags Nothing >>= \ p -> displayPane p False) [] False
-    ,AD "CleanPackage" "Cl_ean Package" (Just "Cleans the package") (Just "ide_clean")
-        (packageClean Nothing) [] False
-    ,AD "ConfigPackage" "_Configure Package" (Just "Configures the package") (Just "ide_configure")
-        packageConfig [] False
-    ,AD "BuildPackage" "_Build Package" (Just "Builds the package") (Just "ide_make")
-        makePackage [] False
+    ,AD "CleanPackage" "Cl_ean" (Just "Cleans the package") (Just "ide_clean")
+        (packageTry_ packageClean) [] False
+    ,AD "ConfigPackage" "_Configure" (Just "Configures the package") (Just "ide_configure")
+        (packageTry_ packageConfig) [] False
+    ,AD "BuildPackage" "_Build" (Just "Builds the package") (Just "ide_make")
+        (packageTry_ makePackage) [] False
     ,AD "DocPackage" "_Build Documentation" (Just "Builds the documentation") Nothing
-        packageDoc [] False
-    ,AD "CopyPackage" "_Copy Package" (Just "Copies the package") Nothing
-        packageCopy [] False
+        (packageTry_ packageDoc) [] False
+    ,AD "CopyPackage" "_Copy" (Just "Copies the package") Nothing
+        (packageTry_ packageCopy) [] False
     ,AD "RunPackage" "_Run" (Just "Runs the package") (Just "ide_run")
-        packageRun [] False
-    ,AD "AddAllImports" "_Add All Imports" (Just "Resolve 'Not in scope' errors by adding the necessary imports") Nothing
-        addAllImports [] False
+        (packageTry_ packageRun) [] False
+    ,AD "ResolveErrors" "Resol_ve Errors" (Just "Resolve 'Hidden package' and 'Not in scope' errors by adding the necessary dependancies or imports") Nothing
+        resolveErrors [] False
 
-    ,AD "InstallPackage" "_Install Package" Nothing Nothing
-        packageInstall [] False
-    ,AD "RegisterPackage" "_Register Package" Nothing Nothing
-        packageRegister [] False
-    ,AD "UnregisterPackage" "_Unregister" Nothing Nothing
-        packageUnregister [] False
-    ,AD "TestPackage" "Test Package" Nothing Nothing
-        packageTest [] False
+    ,AD "InstallPackage" "_Install" Nothing Nothing
+        (packageTry_ packageInstall) [] False
+    ,AD "RegisterPackage" "_Register" Nothing Nothing
+        (packageTry_ packageRegister) [] False
+    ,AD "TestPackage" "Test" Nothing Nothing
+        (packageTry_ packageTest) [] False
     ,AD "SdistPackage" "Source Dist" Nothing Nothing
-        packageSdist [] False
+        (packageTry_ packageSdist) [] False
     ,AD "OpenDocPackage" "_Open Doc" Nothing Nothing
-        packageOpenDoc [] False
+        (packageTry_ packageOpenDoc) [] False
 
     ,AD "Debug" "_Debug" Nothing Nothing (return ()) [] False
     ,AD "StartDebugger" "_Start Debugger" (Just "Starts using the GHCi debugger for build and run") Nothing
-        debugStart [] False
+        (packageTry_ debugStart) [] False
     ,AD "QuitDebugger" "_Quit Debugger" (Just "Quit the GHCi debugger if it is running") Nothing
         debugQuit [] False
     ,AD "ExecuteSelection" "_Execute Selection" (Just "Sends the selected text to the debugger") Nothing
@@ -334,10 +337,8 @@ mkActions =
         (getErrors Nothing  >>= \ p -> displayPane p False) [] False
     ,AD "ShowLog" "Log" Nothing Nothing
         showLog [] False
-    ,AD "ShowReferences" "References" Nothing Nothing
-        (getReferences Nothing >>= \ p -> displayPane p False) [] False
     ,AD "ShowWorkspace" "Workspace" Nothing Nothing
-        (getWorkspace Nothing >>= \ p -> displayPane p False) [] False
+        showWorkspace [] False
 
     ,AD "View" "_View" Nothing Nothing (return ()) [] False
     ,AD "ViewMoveLeft" "Move _Left" Nothing Nothing
@@ -408,10 +409,13 @@ mkActions =
 
     ,AD "BackgroundBuildToggled" "_BackgroundBuild" (Just "Build in the background and report errors") (Just "ide_build")
         backgroundBuildToggled [] True
-    ,AD "BackgroundLinkToggled" "_BackgroundLink" (Just "Link in the background") (Just "ide_link")
-        backgroundLinkToggled [] True
+    ,AD "MakeModeToggled" "_MakeMode" (Just "Make dependent packages") (Just "ide_make")
+        makeModeToggled [] True
     ,AD "DebugToggled" "_Debug" (Just "Use GHCi debugger to build and run") (Just "ide_debug")
-        debugToggled [] True]
+        debugToggled [] True
+    ,AD "OpenDocu" "_OpenDocu" (Just "Opens a browser for a search of the selected data") Nothing
+        openDocu [] True
+        ]
 
 --
 -- | The menu description in XML Syntax as defined by GTK
@@ -448,7 +452,7 @@ updateRecentEntries = do
             fe <- doesFileExist s
             when fe $ do
                 mi <- menuItemNewWithLabel s
-                mi `onActivateLeaf` (reflectIDE (workspaceOpenThis True (Just s) >> return ()) ideR)
+                mi `onActivateLeaf` (reflectIDE (workspaceOpenThis True (Just s) >> showWorkspace) ideR)
                 menuShellAppend recentWorkspacesMenu mi) recentWorkspaces'
         oldSubmenu <- menuItemGetSubmenu recentWorkspacesItem
         when (isJust oldSubmenu) $ do
@@ -561,15 +565,19 @@ textPopupMenu ideR menu = do
     mapM_ widgetShow (castToWidget sep1 : castToWidget sep2 : otherEntries)
     mapM_ widgetHide $ take 2 (reverse items)
 
+canQuit :: IDEM Bool
+canQuit = do
+    modifyIDE_ (\ide -> ide{currentState = IsShuttingDown})
+    saveSession :: IDEAction
+    can <- fileCloseAll (\_ -> return True)
+    unless can $ modifyIDE_ (\ide -> ide{currentState = IsRunning})
+    return can
+
 -- | Quit ide
 quit :: IDEAction
 quit = do
-    modifyIDE_ (\ide -> ide{currentState = IsShuttingDown})
-    saveSession :: IDEAction
-    b <- fileCloseAll (\_ -> return True)
-    if b
-        then liftIO mainQuit
-        else modifyIDE_ (\ide -> ide{currentState = IsRunning})
+    can <- canQuit
+    when can $ liftIO mainQuit
 
 --
 -- | Show the about dialog
@@ -579,7 +587,7 @@ aboutDialog = do
     d <- aboutDialogNew
     aboutDialogSetName d "Leksah"
     aboutDialogSetVersion d (showVersion version)
-    aboutDialogSetCopyright d "Copyright 2007-2010 Jürgen Nicklisch-Franken, Hamish Mackenzie"
+    aboutDialogSetCopyright d "Copyright 2007-2011 Jürgen Nicklisch-Franken, Hamish Mackenzie"
     aboutDialogSetComments d $ "An integrated development environement (IDE) for the " ++
                                "programming language Haskell and the Glasgow Haskell Compiler"
     dd <- getDataDir
@@ -624,7 +632,7 @@ getActionsFor SensitivityBackwardHist = getActionsFor' ["ViewHistoryBack"]
 getActionsFor SensitivityProjectActive = getActionsFor'
     ["EditPackage", "PackageFlags", "ConfigPackage", "BuildPackage"
     ,"DocPackage", "CleanPackage", "CopyPackage", "RunPackage","InstallPackage"
-    ,"RegisterPackage", "UnregisterPackage","TestPackage","SdistPackage"
+    ,"RegisterPackage", "TestPackage","SdistPackage"
     ,"OpenDocPackage","FileCloseAll"]
 getActionsFor SensitivityError = getActionsFor' ["NextError", "PreviousError"]
 getActionsFor SensitivityEditor = getActionsFor' ["EditUndo", "EditRedo", "EditGotoLine"
@@ -684,9 +692,9 @@ instrumentWindow win prefs topWidget = do
         windowAddAccelGroup win acc
         containerAdd win vb
         reflectIDE (do
-            setCandyState (isJust (sourceCandy prefs))
+            setCandyState (fst (sourceCandy prefs))
             setBackgroundBuildToggled (backgroundBuild prefs)
-            setBackgroundLinkToggled (backgroundLink prefs)) ideR
+            setMakeModeToggled (makeMode prefs)) ideR
 
 instrumentSecWindow :: Window -> IDEAction
 instrumentSecWindow win = do
@@ -764,61 +772,66 @@ setSymbol symbol = do
             case filter (not . isReexported) (getIdentifierDescr symbol symbolTable1 symbolTable2) of
                 []     -> return ()
                 a:[]   -> selectIdentifier a
+                a:b:[] -> if isJust (dscMbModu a) && dscMbModu a == dscMbModu b &&
+                            isNear (dscMbLocation a) (dscMbLocation b)
+                                then selectIdentifier a
+                                else setChoices [a,b]
                 l      -> setChoices l
-
+  where
+    isNear (Just a) (Just b) = abs (locationSLine a - locationSLine b) <= 3
+    isNear _ _               = False
 --
 -- | Register handlers for IDE events
 --
-registerEvents :: IDEAction
-registerEvents =    do
+registerLeksahEvents :: IDEAction
+registerLeksahEvents =    do
     stRef   <-  ask
     registerEvent stRef "LogMessage"
-        (Left (\e@(LogMessage s t)      -> getLog >>= \(log :: IDELog) -> liftIO $ appendLog log s t
-                                                >> return e))
+        (\e@(LogMessage s t)      -> getLog >>= \(log :: IDELog) -> liftIO $ appendLog log s t
+                                                >> return e)
     registerEvent stRef "SelectInfo"
-        (Left (\ e@(SelectInfo str)     -> setSymbol str >> return e))
+        (\ e@(SelectInfo str)     -> setSymbol str >> return e)
     registerEvent stRef "SelectIdent"
-        (Left (\ e@(SelectIdent id)     -> selectIdentifier id >> return e))
+        (\ e@(SelectIdent id)     -> selectIdentifier id >> return e)
     registerEvent stRef "InfoChanged"
-        (Left (\ e@(InfoChanged b)      -> reloadKeepSelection b >> return e))
+        (\ e@(InfoChanged b)      -> reloadKeepSelection b >> return e)
     registerEvent stRef "UpdateWorkspaceInfo"
-        (Left (\ e@UpdateWorkspaceInfo  -> updateWorkspaceInfo >> return e))
+        (\ e@UpdateWorkspaceInfo  -> updateWorkspaceInfo >> return e)
     registerEvent stRef "WorkspaceChanged"
-        (Left (\ e@(WorkspaceChanged showPane updateFileCache)
-                                        -> updateWorkspace showPane updateFileCache >> return e))
+        (\ e@(WorkspaceChanged showPane updateFileCache)
+                                        -> updateWorkspace showPane updateFileCache >> return e)
     registerEvent stRef "RecordHistory"
-        (Left (\ rh@(RecordHistory h)   -> recordHistory h >> return rh))
+        (\ rh@(RecordHistory h)   -> recordHistory h >> return rh)
     registerEvent stRef "Sensitivity"
-        (Left (\ s@(Sensitivity h)      -> setSensitivity h >> return s))
+        (\ s@(Sensitivity h)      -> setSensitivity h >> return s)
     registerEvent stRef "SearchMeta"
-        (Left (\ e@(SearchMeta string)  -> searchMetaGUI string >> return e))
+        (\ e@(SearchMeta string)  -> searchMetaGUI string >> return e)
     registerEvent stRef "LoadSession"
-        (Left (\ e@(LoadSession fp)     -> loadSession fp >> return e))
+        (\ e@(LoadSession fp)     -> loadSession fp >> return e)
     registerEvent stRef "SaveSession"
-        (Left (\ e@(SaveSession fp)     -> saveSessionAs fp Nothing >> return e))
+        (\ e@(SaveSession fp)     -> saveSessionAs fp Nothing >> return e)
     registerEvent stRef "UpdateRecent"
-        (Left (\ e@UpdateRecent         -> updateRecentEntries >> return e))
+        (\ e@UpdateRecent         -> updateRecentEntries >> return e)
     registerEvent stRef "VariablesChanged"
-        (Left (\ e@VariablesChanged     -> fillVariablesList >> return e))
+        (\ e@VariablesChanged     -> fillVariablesList >> return e)
     registerEvent stRef "ErrorChanged"
-        (Left (\ e@ErrorChanged         -> postAsyncIDE fillErrorList >> return e))
+        (\ e@ErrorChanged         -> postAsyncIDE fillErrorList >> return e)
     registerEvent stRef "CurrentErrorChanged"
-        (Left (\ e@(CurrentErrorChanged mbLogRef) -> postAsyncIDE (do
+        (\ e@(CurrentErrorChanged mbLogRef) -> postAsyncIDE (do
             selectRef mbLogRef
-            selectError mbLogRef)  >> return e))
+            selectError mbLogRef)  >> return e)
     registerEvent stRef "BreakpointChanged"
-        (Left (\ e@BreakpointChanged    -> postAsyncIDE fillBreakpointList >> return e))
+        (\ e@BreakpointChanged    -> postAsyncIDE fillBreakpointList >> return e)
     registerEvent stRef "CurrentBreakChanged"
-        (Left (\ e@(CurrentBreakChanged mbLogRef) -> postAsyncIDE (do
+        (\ e@(CurrentBreakChanged mbLogRef) -> postAsyncIDE (do
             selectRef mbLogRef
-            selectBreak mbLogRef) >> return e))
+            selectBreak mbLogRef) >> return e)
     registerEvent stRef "TraceChanged"
-        (Left (\ e@TraceChanged         -> fillTraceList >> return e))
+        (\ e@TraceChanged         -> fillTraceList >> return e)
     registerEvent stRef "GetTextPopup"
-        (Left (\ e@(GetTextPopup _)     -> return (GetTextPopup (Just textPopupMenu))))
+        (\ e@(GetTextPopup _)     -> return (GetTextPopup (Just textPopupMenu)))
     registerEvent stRef "StatusbarChanged"
-        (Left (\ e@(StatusbarChanged args)
-                                        -> changeStatusbar args >> return e))
+        (\ e@(StatusbarChanged args) -> changeStatusbar args >> return e)
     return ()
 
 
