@@ -32,6 +32,19 @@ module IDE.Core.Types (
 ,   IDEM
 ,   IDEAction
 ,   IDEEvent(..)
+,   liftIDE
+
+,   WorkspaceM
+,   WorkspaceAction
+,   runWorkspace
+
+,   PackageM
+,   PackageAction
+,   runPackage
+
+,   DebugM
+,   DebugAction
+,   runDebug
 
 ,   IDEPackage(..)
 ,   Workspace(..)
@@ -41,7 +54,6 @@ module IDE.Core.Types (
 ,   KeyString
 
 ,   Prefs(..)
-,   InstallFlag(..)
 
 ,   LogRefType(..)
 ,   LogRef(..)
@@ -72,9 +84,7 @@ module IDE.Core.Types (
 ,   StatusbarCompartment(..)
 ) where
 
-#ifdef YI
-import Yi.UI.Pango.Control as Yi
-#endif
+import qualified IDE.YiConfig as Yi
 import Graphics.UI.Gtk
        (Window(..), KeyVal(..), Color(..), Menu(..), TreeView(..),
         ListStore(..), Toolbar(..))
@@ -99,10 +109,12 @@ import Data.IORef (writeIORef, readIORef, IORef(..))
 import Numeric (showHex)
 import Control.Event
     (EventSelector(..), EventSource(..), Event(..))
-import System.FilePath ((</>))
+import System.FilePath (dropFileName, (</>))
 import IDE.Core.CTypes
 import IDE.StrippedPrefs(RetrieveStrategy)
 import System.IO (Handle)
+import Distribution.Text(disp)
+import Text.PrettyPrint (render)
 
 -- ---------------------------------------------------------------------
 -- IDE State
@@ -136,11 +148,9 @@ data IDE            =  IDE {
 ,   recentFiles     ::   [FilePath]
 ,   recentWorkspaces ::  [FilePath]
 ,   runningTool     ::   Maybe ProcessHandle
-,   ghciState       ::   Maybe ToolState
-,   completion      ::   Maybe CompletionWindow
-#ifdef YI
+,   debugState      ::   Maybe (IDEPackage, ToolState)
+,   completion      ::   ((Int, Int), Maybe CompletionWindow)
 ,   yiControl       ::   Yi.Control
-#endif
 ,   server          ::   Maybe Handle
 } --deriving Show
 
@@ -174,6 +184,36 @@ data IDEState =
     |   IsCompleting Connections
 
 
+liftIDE :: IDEM a -> WorkspaceM a
+liftIDE = lift
+
+-- ---------------------------------------------------------------------
+-- Monad for functions that need an open workspace
+--
+type WorkspaceM = ReaderT Workspace IDEM
+type WorkspaceAction = WorkspaceM ()
+
+runWorkspace :: WorkspaceM a -> Workspace -> IDEM a
+runWorkspace = runReaderT
+
+-- ---------------------------------------------------------------------
+-- Monad for functions that need an active package
+--
+type PackageM = ReaderT IDEPackage IDEM
+type PackageAction = PackageM ()
+
+runPackage :: PackageM a -> IDEPackage -> IDEM a
+runPackage = runReaderT
+
+-- ---------------------------------------------------------------------
+-- Monad for functions that need to use the GHCi debugger
+--
+type DebugM = ReaderT (IDEPackage, ToolState) IDEM
+type DebugAction = DebugM ()
+
+runDebug :: DebugM a -> (IDEPackage, ToolState) -> IDEM a
+runDebug = runReaderT
+
 -- ---------------------------------------------------------------------
 -- Events which can be signalled and handled
 --
@@ -277,7 +317,10 @@ data IDEPackage     =   IDEPackage {
 ,   ipdUnregisterFlags ::   [String]
 ,   ipdSdistFlags      ::   [String]
 }
-    deriving (Eq,Show)
+    deriving (Eq)
+
+instance Show IDEPackage where
+    show p = show "IDEPackage for " ++ (render . disp) (ipdPackageId p)
 
 instance Ord IDEPackage where
     compare x y     =   compare (ipdPackageId x) (ipdPackageId y)
@@ -291,10 +334,9 @@ data Workspace = Workspace {
 ,   wsName          ::   String
 ,   wsFile          ::   FilePath
 ,   wsPackages      ::   [IDEPackage]
-,   wsActivePack    ::   Maybe IDEPackage
 ,   wsPackagesFiles ::   [FilePath]
 ,   wsActivePackFile::   Maybe FilePath
-,   wsReverseDeps   ::   Map IDEPackage [IDEPackage]
+,   wsNobuildPack   ::   [IDEPackage]
 } deriving Show
 
 -- ---------------------------------------------------------------------
@@ -325,17 +367,19 @@ data Prefs = Prefs {
         prefsFormat         ::   Int
     ,   prefsSaveTime       ::   String
     ,   showLineNumbers     ::   Bool
-    ,   rightMargin         ::   Maybe Int
+    ,   rightMargin         ::   (Bool, Int)
     ,   tabWidth            ::   Int
-    ,   sourceCandy         ::   Maybe String
+    ,   wrapLines           ::   Bool
+    ,   sourceCandy         ::   (Bool,String)
     ,   keymapName          ::   String
     ,   forceLineEnds       ::   Bool
     ,   removeTBlanks       ::   Bool
     ,   textviewFont        ::   Maybe String
-    ,   sourceStyle         ::   Maybe String
+    ,   sourceStyle         ::   (Bool, String)
     ,   foundBackground     ::   Color
     ,   contextBackground   ::   Color
     ,   breakpointBackground ::  Color
+    ,   autoLoad            ::   Bool
     ,   useYi               ::   Bool
     ,   logviewFont         ::   Maybe String
     ,   defaultSize         ::   (Int,Int)
@@ -350,9 +394,10 @@ data Prefs = Prefs {
     ,   completeRestricted  ::   Bool
     ,   saveAllBeforeBuild  ::   Bool
     ,   backgroundBuild     ::   Bool
-    ,   backgroundLink      ::   Bool
+    ,   makeMode            ::   Bool
+    ,   singleBuildWithoutLinking :: Bool
+    ,   dontInstallLast     ::   Bool
     ,   printEvldWithShow   ::   Bool
-    ,   autoInstall         ::   InstallFlag
     ,   breakOnException    ::   Bool
     ,   breakOnError        ::   Bool
     ,   printBindResult     ::   Bool
@@ -369,9 +414,6 @@ data Prefs = Prefs {
 data SearchHint = Forward | Backward | Insert | Delete | Initial
     deriving (Eq)
 
-data InstallFlag        = InstallAlways | InstallLibs | InstallNo
-    deriving(Eq,Show, Read, Enum, Bounded)
-
 instance Ord Modifier
     where compare a b = compare (fromEnum a) (fromEnum b)
 
@@ -382,7 +424,7 @@ data LogRefType = WarningRef | ErrorRef | BreakpointRef | ContextRef deriving (E
 
 data LogRef = LogRef {
     logRefSrcSpan       ::   SrcSpan
-,   logRefRootPath      ::   FilePath
+,   logRefPackage       ::   IDEPackage
 ,   refDescription      ::   String
 ,   logLines            ::   (Int,Int)
 ,   logRefType          ::   LogRefType
@@ -400,6 +442,8 @@ displaySrcSpan s = srcSpanFilename s ++ ":" ++
         else show (srcSpanStartLine s) ++ ":" ++
             show (srcSpanStartColumn s) ++ "-" ++ show (srcSpanEndColumn s)
 
+logRefRootPath :: LogRef -> FilePath
+logRefRootPath = dropFileName . ipdCabalFile . logRefPackage
 
 logRefFilePath :: LogRef -> FilePath
 logRefFilePath = srcSpanFilename . logRefSrcSpan
