@@ -66,14 +66,10 @@ import Data.Char (digitToInt, isDigit, toLower, isAlphaNum)
 import Text.Regex.TDFA hiding (caseSensitive)
 import qualified Text.Regex.TDFA as Regex
 import Text.Regex.TDFA.String (compile)
-import Data.List (nub, find, isPrefixOf)
+import Data.List (find, isPrefixOf)
 import Data.Array (bounds, (!), inRange)
-import IDE.Utils.Tool (runTool)
-import Control.Concurrent (forkIO)
-import IDE.Pane.Grep
-import IDE.Package (getPackageDescriptionAndPath)
-import Distribution.PackageDescription (allBuildInfo, hsSourceDirs)
-import System.FilePath (dropFileName)
+import IDE.Pane.Grep (grepWorkspace)
+import IDE.Workspaces (workspaceTry_, packageTry_)
 
 data FindState = FindState {
             entryStr        ::    String
@@ -205,7 +201,7 @@ constructFindReplace = reifyIDE $ \ ideR   -> do
 
     grepButton <- toolButtonNew (Nothing :: Maybe Widget) (Just "Grep")
     toolbarInsert toolbar grepButton 0
-    grepButton `onToolButtonClicked` (reflectIDE (doGrep toolbar) ideR)
+    grepButton `onToolButtonClicked` (reflectIDE (packageTry_ $ doGrep toolbar) ideR)
     tooltipsSetTip tooltips grepButton "Search in multiple files" ""
 
     sep1 <- separatorToolItemNew
@@ -289,7 +285,7 @@ constructFindReplace = reifyIDE $ \ ideR   -> do
     tooltipsSetTip tooltips entireWordButton "When selected only entire words are matched" ""
 
     caseSensitiveButton <- toggleToolButtonNew
-    toolButtonSetLabel caseSensitiveButton (Just "c. S.")
+    toolButtonSetLabel caseSensitiveButton (Just "Case")
     widgetSetName caseSensitiveButton "caseSensitiveButton"
     toolbarInsert toolbar caseSensitiveButton 0
     caseSensitiveButton `onToolButtonClicked` (doSearch toolbar Insert ideR)
@@ -390,9 +386,10 @@ doSearch fb hint ideR   = do
     reflectIDE (addToHist search) ideR
     return ()
 
-doGrep :: Toolbar -> IDEAction
+doGrep :: Toolbar -> PackageAction
 doGrep fb   = do
-    ideR          <- ask
+    package       <- ask
+    ideR          <- lift $ ask
     entry         <- liftIO $ getFindEntry fb
     search        <- liftIO $ entryGetText (castToEntry entry)
     entireWord    <- liftIO $ getEntireWord fb
@@ -400,17 +397,7 @@ doGrep fb   = do
     wrapAround    <- liftIO $ getWrapAround fb
     regex         <- liftIO $ getRegex fb
     let (regexString, _) = regexStringAndMatchIndex entireWord regex search
-    mbPD <- getPackageDescriptionAndPath
-    case mbPD of
-        Nothing             -> ideMessage Normal "No package description"
-        Just (pd,cabalPath) -> do
-            let srcPaths = nub $ concatMap hsSourceDirs $ allBuildInfo pd
-            let dir = dropFileName (cabalPath)
-            liftIO $ forkIO $ do
-                (output, pid) <- runTool "grep" ((if caseSensitive then [] else ["-i"])
-                    ++ ["-r", "-E", "-n", "--exclude=*~", regexString] ++ srcPaths) (Just dir)
-                reflectIDE (setGrepResults output) ideR
-            return ()
+    lift $ workspaceTry_ $ grepWorkspace regexString caseSensitive
 
 matchFunc :: ListStore String -> String -> TreeIter -> IO Bool
 matchFunc model str iter = do
