@@ -1,7 +1,7 @@
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.ImportTool
--- Copyright   :  2007-2010 Juergen Nicklisch-Franken, Hamish Mackenzie
+-- Copyright   :  2007-2011 Juergen Nicklisch-Franken, Hamish Mackenzie
 -- License     :  GPL
 --
 -- Maintainer  :  Jutaro <jutaro@leksah.org>
@@ -13,10 +13,12 @@
 -----------------------------------------------------------------------------
 
 module IDE.ImportTool (
-    addAllImports
+    resolveErrors
 ,   addOneImport
 ,   addImport
+,   addPackage
 ,   parseNotInScope
+,   parseHiddenModule
 ) where
 
 import IDE.Core.State
@@ -24,10 +26,8 @@ import Data.Maybe (isNothing,isJust)
 import IDE.Metainfo.Provider
        (getPackageImportInfo, getIdentifierDescr)
 import Text.PrettyPrint (render)
-import Distribution.Text (disp)
+import Distribution.Text (simpleParse, display, disp)
 import IDE.Pane.SourceBuffer
-       (belongsToPackage, maybeActiveBuf, fileSave,
-        inActiveBufContext, selectSourceBuf)
 import Graphics.UI.Gtk
 import Text.ParserCombinators.Parsec.Language (haskellStyle)
 import Graphics.UI.Editor.MakeEditor
@@ -39,8 +39,7 @@ import Data.Maybe (fromJust)
 import Text.ParserCombinators.Parsec hiding (parse)
 import qualified Text.ParserCombinators.Parsec as Parsec (parse)
 import Graphics.UI.Editor.Simple (staticListEditor)
-import Control.Monad (when)
-import Distribution.Text (display)
+import Control.Monad (forM, when)
 import Data.List (sort, nub, nubBy)
 import IDE.Utils.ServerConnection
 import Text.PrinterParser (prettyPrint)
@@ -49,25 +48,32 @@ import qualified Distribution.ModuleName as D (ModuleName(..))
 import qualified Text.ParserCombinators.Parsec.Token as P
        (operator, dot, identifier, symbol, lexeme, whiteSpace,
         makeTokenParser)
-import Graphics.UI.Gtk.Gdk.Events (Event(..))
 import Control.Monad.Trans (liftIO)
-
-
-
+import Distribution.PackageDescription.Parse
+       (writePackageDescription, readPackageDescription)
+import Distribution.Verbosity (normal)
+import IDE.Pane.PackageEditor (hasConfigs)
+import Distribution.Package
+import Distribution.Version (VersionRange(..))
+import Distribution.PackageDescription (buildDepends)
+import Distribution.PackageDescription.Configuration
+       (flattenPackageDescription)
 
 -- | Add all imports which gave error messages ...
-addAllImports :: IDEAction
-addAllImports = do
+resolveErrors :: IDEAction
+resolveErrors = do
     prefs' <- readIDE prefs
     let buildInBackground = backgroundBuild prefs'
     when buildInBackground $
         modifyIDE_ (\ide -> ide{prefs = prefs'{backgroundBuild = False}})
     errors <- readIDE errorRefs
-    addAll buildInBackground
-        [ y | (x,y) <-
+    addPackageResults <- forM errors addPackage
+    let notInScopes = [ y | (x,y) <-
             nubBy (\ (p1,_) (p2,_) -> p1 == p2)
                 $ [(x,y) |  (x,y) <- [((parseNotInScope . refDescription) e, e) | e <- errors]],
-                                isJust x] (True,[])
+                                isJust x]
+    when (not (or addPackageResults) && null notInScopes) $ ideMessage Normal $ "No errors that can be auto resolved"
+    addAll buildInBackground notInScopes (True,[])
     where
         addAll :: Bool -> [LogRef] -> (Bool,[Descr]) -> IDEM ()
         addAll bib (errorSpec:rest) (True,descrList)  =  addImport errorSpec descrList (addAll bib rest)
@@ -86,8 +92,7 @@ addOneImport = do
         Nothing -> do
             ideMessage Normal $ "No error selected"
             return ()
-        Just ref -> do
-            addImport ref [] (\ _ -> return ())
+        Just ref -> addImport ref [] (\ _ -> return ())
 
 -- | Add one missing import
 -- Returns a boolean, if the process should be stopped in case of multiple addition
@@ -122,6 +127,23 @@ addImport error descrList continuation =
                                                     else addImport' nis (logRefFullFilePath error)
                                                             descr descrList continuation
 
+addPackage :: LogRef -> IDEM Bool
+addPackage error = do
+    case parseHiddenModule (refDescription error) of
+        Nothing -> return False
+        Just (HiddenModuleResult mod pack) -> do
+            let idePackage = logRefPackage error
+            package <- liftIO $ readPackageDescription normal (ipdCabalFile $ idePackage)
+            if hasConfigs package
+                then return False
+                else do
+                    let flat = flattenPackageDescription package
+                    ideMessage Normal $ "addPackage " ++ (display $ pkgName pack)
+                    liftIO $ writePackageDescription (ipdCabalFile $ idePackage)
+                        flat { buildDepends =
+                            Dependency (pkgName pack) AnyVersion : buildDepends flat}
+                    return True
+
 getScopeForActiveBuffer :: IDEM (Maybe (GenScope, GenScope))
 getScopeForActiveBuffer = do
     mbActiveBuf <- maybeActiveBuf
@@ -320,28 +342,18 @@ selectModuleDialog parentWindow list id mbQual mbDescr =
             dia               <- dialogNew
             windowSetTransientFor dia parentWindow
             upper             <- dialogGetUpper dia
-            dialogAddButton dia "Ok" ResponseOk
+            okButton <- dialogAddButton dia "Ok" ResponseOk
             dialogAddButton dia "Cancel" ResponseCancel
             (widget,inj,ext,_) <- buildEditor (moduleFields selectionList qualId) realSelectionString
             boxPackStart upper widget PackGrow 7
             dialogSetDefaultResponse dia ResponseOk --does not work for the tree view
             widgetShowAll dia
             rw <- getRealWidget widget
---            case rw of
---                Nothing -> return ()
---                Just r -> do
---                    r `onKeyPress` \ event -> do
---                        case event of
---                            Key { eventKeyName = name} ->
---                                case name of
---                                    "Return" -> do
---                                        dialogResponse dia ResponseOk
---                                        return True
---                                    _        -> return False
---                            _ -> return False
---                    return ()
+            set okButton [widgetCanDefault := True]
+            widgetGrabDefault okButton
             resp <- dialogRun dia
             value                      <- ext ([])
+            widgetHide dia
             widgetDestroy dia
             --find
             case (resp,value) of
@@ -351,3 +363,37 @@ selectModuleDialog parentWindow list id mbQual mbDescr =
                                                 Just pm -> (render . disp . modu) pm == v) list)))
                 _                      -> return Nothing
 
+--testString =    "    Could not find module `Graphics.UI.Gtk':\n"
+--             ++ "      It is a member of the hidden package `gtk-0.11.0'.\n"
+--             ++ "      Perhaps you need to add `gtk' to the build-depends in your .cabal file.\n"
+--             ++ "      Use -v to see a list of the files searched for."
+--
+--test = parseHiddenModule testString == Just (HiddenModuleResult {hiddenModule = "Graphics.UI.Gtk", missingPackage = PackageIdentifier {pkgName = PackageName "gtk", pkgVersion = Version {versionBranch = [0,11,0], versionTags = []}}})
+
+data HiddenModuleResult = HiddenModuleResult {
+        hiddenModule      :: String
+    ,   missingPackage    :: PackageId}
+    deriving (Eq, Show)
+
+parseHiddenModule :: String -> (Maybe HiddenModuleResult)
+parseHiddenModule str =
+    case Parsec.parse hiddenModuleParser "" str of
+        Left e             -> Nothing
+        Right (mod, pack)  ->
+            case simpleParse pack of
+                Just p  -> Just $ HiddenModuleResult mod p
+                Nothing -> Nothing
+
+hiddenModuleParser :: CharParser () (String, String)
+hiddenModuleParser = do
+    whiteSpace
+    symbol "Could not find module `"
+    mod    <- many (noneOf "'")
+    symbol "':\n"
+    whiteSpace
+    symbol "It is a member of the hidden package `"
+    pack   <- many (noneOf "'")
+    symbol "'.\n"
+    many anyChar
+    return (mod, pack)
+    <?> "hiddenModuleParser"
