@@ -1,8 +1,7 @@
-
-{-# OPTIONS_GHC -XScopedTypeVariables #-}
+{-# LANGUAGE CPP, ScopedTypeVariables #-}
 -----------------------------------------------------------------------------
 --
--- Module      :  Main
+-- Module      :  IDE.Leksah
 -- Copyright   :  (c) Juergen Nicklisch-Franken, Hamish Mackenzie
 -- License     :  GNU-GPL
 --
@@ -15,8 +14,8 @@
 --
 ---------------------------------------------------------------------------------
 
-module Main (
-    main
+module IDE.Leksah (
+    leksah
 ) where
 
 import Graphics.UI.Gtk
@@ -30,13 +29,14 @@ import System.Environment
 import Data.Version
 import Prelude hiding(catch)
 
-#if defined(darwin_HOST_OS)
-import IDE.OSX
-#endif
+import qualified IDE.OSX as OSX
+import qualified IDE.YiConfig as Yi
 
-#ifdef YI
-import qualified Yi.UI.Pango.Control as Yi
-import IDE.YiConfig
+#ifdef LEKSAH_WITH_YI_DYRE
+import System.Directory (getAppUserDataDirectory)
+import System.FilePath ((</>))
+import Control.Applicative ((<$>))
+import qualified Config.Dyre as Dyre
 #endif
 
 import Paths_leksah
@@ -54,9 +54,9 @@ import IDE.Pane.SourceBuffer
 import IDE.Find
 import Graphics.UI.Editor.Composite (filesEditor, maybeEditor)
 import Graphics.UI.Editor.Simple
-       (enumEditor, intEditor, stringEditor, boolEditor)
+       (enumEditor, stringEditor)
 import IDE.Metainfo.Provider (initInfo)
-import IDE.Workspaces (backgroundMake)
+import IDE.Workspaces (workspaceOpenThis, backgroundMake)
 import IDE.Utils.GUIUtils
 import Network (withSocketsDo)
 import Control.Exception
@@ -64,32 +64,37 @@ import System.Exit(exitFailure)
 import qualified IDE.StrippedPrefs as SP
 import IDE.Utils.Tool (runTool,toolline)
 import IDE.System.Process(waitForProcess)
-#if defined(mingw32_HOST_OS) || defined(__MINGW32__)
-#else
-import qualified System.Posix as P
-#endif
 import System.Log
 import System.Log.Logger(updateGlobalLogger,rootLoggerName,setLevel)
 import Data.List (stripPrefix)
+import System.Directory (doesFileExist)
+import System.FilePath (dropExtension, splitExtension, (</>))
 
 -- --------------------------------------------------------------------
 -- Command line options
 --
 
-data Flag =  VersionF | SessionN String | Help | Verbosity String
+data Flag =  VersionF | SessionN String | EmptySession | DefaultSession | Help | Verbosity String
        deriving (Show,Eq)
 
 options :: [OptDescr Flag]
-options =   [Option ['v'] ["version"] (NoArg VersionF)
-                "Show the version number of ide"
+options =   [Option ['e'] ["emptySession"] (NoArg EmptySession)
+                "Start with empty session"
+         ,   Option ['d'] ["defaultSession"] (NoArg DefaultSession)
+                "Start with default session (can be used together with a source file)"
          ,   Option ['l'] ["loadSession"] (ReqArg SessionN "NAME")
                 "Load session"
+
          ,   Option ['h'] ["help"] (NoArg Help)
                 "Display command line options"
-         ,   Option ['e'] ["verbosity"] (ReqArg Verbosity "Verbosity")
-                "One of DEBUG, INFO, NOTICE, WARNING, ERROR, CRITICAL, ALERT, EMERGENCY"]
+         ,   Option ['v'] ["version"] (NoArg VersionF)
+                "Show the version number of ide"
 
-header = "Usage: leksah [OPTION...] files..."
+         ,   Option ['e'] ["verbosity"] (ReqArg Verbosity "Verbosity")
+             "One of DEBUG, INFO, NOTICE, WARNING, ERROR, CRITICAL, ALERT, EMERGENCY"]
+
+
+header = "Usage: leksah [OPTION...] [file(.lkshs|.lkshw|.hs|.lhs)]"
 
 ideOpts :: [String] -> IO ([Flag], [String])
 ideOpts argv =
@@ -97,26 +102,74 @@ ideOpts argv =
           (o,n,[]  ) -> return (o,n)
           (_,_,errs) -> ioError $ userError $ concat errs ++ usageInfo header options
 
-
 -- ---------------------------------------------------------------------
 -- | Main function
 --
 
-main = withSocketsDo $ handleExceptions $ do
-#if defined(mingw32_HOST_OS) || defined(__MINGW32__)
+#ifdef LEKSAH_WITH_YI_DYRE
+leksahDriver = Dyre.wrapMain Dyre.defaultParams
+    { Dyre.projectName  = "yi"
+    , Dyre.realMain     = \(config, mbError) -> do
+        case mbError of
+            Just error -> putStrLn $ "Error in yi configuration file : " ++ error
+            Nothing    -> return ()
+        realMain config
+    , Dyre.showError    = \(config, _) error -> (config, Just error)
+    , Dyre.configDir    = Just . getAppUserDataDirectory $ "yi"
+    , Dyre.cacheDir     = Just $ ((</> "leksah") <$> (getAppUserDataDirectory "cache"))
+    , Dyre.hidePackages = ["mtl"]
+    , Dyre.ghcOpts = ["-DLEKSAH"]
+    }
+
+leksah yiConfig = leksahDriver (yiConfig, Nothing)
 #else
-    P.getProcessID >>= P.createProcessGroup
+leksah = realMain
 #endif
+
+realMain yiConfig = do
+  withSocketsDo $ handleExceptions $ do
+    dataDir         <- getDataDir
     args            <-  getArgs
 
-    (o,_)           <-  ideOpts args
-    isFirstStart    <-  liftM not hasConfigDir
+    (o,files)       <-  ideOpts args
+    isFirstStart    <-  liftM not $ hasSavedConfigFile standardPreferencesFilename
     let sessions        =   filter (\x -> case x of
                                         SessionN _ -> True
                                         _         -> False) o
-    let sessionFilename =  if not (null sessions)
-                                then  (head $ map (\ (SessionN x) -> x) sessions) ++ leksahSessionFileExtension
-                                else  standardSessionFilename
+
+    let sessionFPs    =   filter (\f -> snd (splitExtension f) == leksahSessionFileExtension) files
+    let workspaceFPs  =   filter (\f -> snd (splitExtension f) == leksahWorkspaceFileExtension) files
+    let sourceFPs     =   filter (\f -> let (_,s) = splitExtension f
+                                        in s == ".hs" ||  s == ".lhs" || s == ".chs") files
+    let mbWorkspaceFP'=  case workspaceFPs of
+                                [] ->  Nothing
+                                w:_ -> Just w
+    (mbWSessionFP, mbWorkspaceFP) <-
+        case mbWorkspaceFP' of
+            Nothing ->  return (Nothing,Nothing)
+            Just fp ->  let spath =  dropExtension fp ++ leksahSessionFileExtension
+                        in do
+                            exists <- liftIO $ doesFileExist spath
+                            if exists
+                                then return (Just spath,Nothing)
+                                else return (Nothing,Just fp)
+    let ssession =  if not (null sessions)
+                                then  (head $ map (\ (SessionN x) -> x) sessions) ++
+                                            leksahSessionFileExtension
+                                else  if null sourceFPs
+                                        then standardSessionFilename
+                                        else emptySessionFilename
+
+    sessionFP    <-  if  elem EmptySession o
+                                then getConfigFilePathForLoad
+                                                        emptySessionFilename Nothing dataDir
+                                else if elem DefaultSession o
+                                        then getConfigFilePathForLoad
+                                                        standardSessionFilename Nothing dataDir
+                                        else case mbWSessionFP of
+                                                Just fp -> return fp
+                                                Nothing -> getConfigFilePathForLoad
+                                                                    ssession Nothing dataDir
     let verbosity'      =  catMaybes $
                                 map (\x -> case x of
                                     Verbosity s -> Just s
@@ -129,11 +182,11 @@ main = withSocketsDo $ handleExceptions $ do
         (sysMessage Normal $ "Leksah the Haskell IDE, version " ++ showVersion version)
     when (elem Help o)
         (sysMessage Normal $ "Leksah the Haskell IDE " ++ usageInfo header options)
-    dataDir         <- getDataDir
+
     prefsPath       <- getConfigFilePathForLoad standardPreferencesFilename Nothing dataDir
     prefs           <- readPrefs prefsPath
     when (not (elem VersionF o) && not (elem Help o))
-        (startGUI sessionFilename prefs isFirstStart)
+        (startGUI yiConfig sessionFP mbWorkspaceFP sourceFPs  prefs isFirstStart)
 
 handleExceptions inner =
   catch inner (\(exception :: SomeException) -> do
@@ -144,36 +197,44 @@ handleExceptions inner =
 -- ---------------------------------------------------------------------
 -- | Start the GUI
 
-startGUI :: String -> Prefs -> Bool -> IO ()
-startGUI sessionFilename iprefs isFirstStart = do
-#ifdef YI
-  Yi.startControl yiVimConfig $ do
-   yiControl <- Yi.getControl
-   Yi.controlIO $ do
-#endif
+startGUI :: Yi.Config -> FilePath -> Maybe FilePath -> [FilePath] -> Prefs -> Bool -> IO ()
+startGUI yiConfig sessionFP mbWorkspaceFP sourceFPs iprefs isFirstStart = do
+  Yi.start yiConfig $ \yiControl -> do
     st          <-  unsafeInitGUIForThreadedRTS
     when rtsSupportsBoundThreads
         (sysMessage Normal "Linked with -threaded")
     timeoutAddFull (yield >> return True) priorityDefaultIdle 100 -- maybe switch back to priorityHigh/???
     mapM_ (sysMessage Normal) st
     initGtkRc
+    dataDir       <- getDataDir
+    mbStartupPrefs <- if not isFirstStart
+                                then return $ Just iprefs
+                                else do
+                                    firstStartOK <- firstStart iprefs
+                                    if not firstStartOK
+                                        then return Nothing
+                                        else do
+                                            prefsPath  <- getConfigFilePathForLoad standardPreferencesFilename Nothing dataDir
+                                            prefs <- readPrefs prefsPath
+                                            return $ Just prefs
+    case mbStartupPrefs of
+        Nothing           -> return ()
+        Just startupPrefs -> startMainWindow yiControl sessionFP mbWorkspaceFP sourceFPs
+                                startupPrefs isFirstStart
+
+startMainWindow :: Yi.Control -> FilePath -> Maybe FilePath -> [FilePath] ->
+                        Prefs -> Bool -> IO ()
+startMainWindow yiControl sessionFP mbWorkspaceFP sourceFPs startupPrefs isFirstStart = do
+    osxApp <- OSX.applicationNew
     uiManager   <-  uiManagerNew
     newIcons
     dataDir       <- getDataDir
-    startupPrefs  <-   if not isFirstStart
-                                then return iprefs
-                                else do
-                                    firstStart iprefs
-                                    prefsPath  <- getConfigFilePathForLoad standardPreferencesFilename Nothing dataDir
-                                    prefs <- readPrefs prefsPath
-                                    return prefs
     candyPath   <-  getConfigFilePathForLoad
                         (case sourceCandy startupPrefs of
-                            Nothing     ->   standardCandyFilename
-                            Just name   ->   name ++ leksahCandyFileExtension) Nothing dataDir
+                            (_,name)   ->   name ++ leksahCandyFileExtension) Nothing dataDir
     candySt     <-  parseCandy candyPath
     -- keystrokes
-    keysPath    <-  getConfigFilePathForLoad (keymapName iprefs ++ leksahKeymapFileExtension) Nothing dataDir
+    keysPath    <-  getConfigFilePathForLoad (keymapName startupPrefs ++ leksahKeymapFileExtension) Nothing dataDir
     keyMap      <-  parseKeymap keysPath
     let accelActions = setKeymap (keyMap :: KeymapI) mkActions
     specialKeys <-  buildSpecialKeys keyMap accelActions
@@ -215,11 +276,9 @@ startGUI sessionFilename iprefs isFirstStart = do
           ,   recentFiles       =   []
           ,   recentWorkspaces  =   []
           ,   runningTool       =   Nothing
-          ,   ghciState         =   Nothing
-          ,   completion        =   Nothing
-#ifdef YI
+          ,   debugState        =   Nothing
+          ,   completion        =   ((750,400),Nothing)
           ,   yiControl         =   yiControl
-#endif
           ,   server            =   Nothing
     }
     ideR             <-  newIORef ide
@@ -231,25 +290,22 @@ startGUI sessionFilename iprefs isFirstStart = do
     win `onDelete` (\ _ -> do reflectIDE quit ideR; return True)
     reflectIDE (instrumentWindow win startupPrefs (castToWidget nb)) ideR
     reflectIDE (do
-        setCandyState (isJust (sourceCandy startupPrefs))
+        setCandyState (fst (sourceCandy startupPrefs))
         setBackgroundBuildToggled (backgroundBuild startupPrefs)
-        setBackgroundLinkToggled (backgroundLink startupPrefs)) ideR
+        setMakeModeToggled (makeMode startupPrefs)) ideR
     let (x,y)   =   defaultSize startupPrefs
     windowSetDefaultSize win x y
-    sessionPath <- getConfigFilePathForLoad sessionFilename Nothing dataDir
     (tbv,fbv)   <- reflectIDE (do
-        registerEvents
-        pair <- recoverSession sessionPath
+        registerLeksahEvents
+        pair <- recoverSession sessionFP
+        workspaceOpenThis False  mbWorkspaceFP
+        mapM fileOpenThis sourceFPs
         wins <- getWindows
         mapM_ instrumentSecWindow (tail wins)
         return pair
         ) ideR
     widgetShowAll win
 
-#if defined(darwin_HOST_OS)
-    updateMenu uiManager
-#endif
-
     reflectIDE (do
         triggerEventIDE UpdateRecent
         if tbv
@@ -257,7 +313,10 @@ startGUI sessionFilename iprefs isFirstStart = do
             else hideToolbar
         if fbv
             then showFindbar
-            else hideFindbar) ideR
+            else hideFindbar
+        OSX.updateMenu osxApp uiManager) ideR
+
+    OSX.applicationReady osxApp
 
     when isFirstStart $ do
         welcomePath <- getConfigFilePathForLoad "welcome.txt" Nothing dataDir
@@ -281,53 +340,48 @@ fDescription configPath = VFD emptyParams [
             (\b a -> a{sourceDirectories = b})
             (filesEditor Nothing FileChooserActionSelectFolder "Select folders")
     ,   mkField
-            (paraName <<<- ParaName "Maybe a directory for unpacking cabal packages" $ emptyParams)
+            (paraName <<<- ParaName "Unpack source for cabal packages to" $ emptyParams)
             unpackDirectory
             (\b a -> a{unpackDirectory = b})
-            (maybeEditor (stringEditor (\ _ -> True),emptyParams) True "")
+            (maybeEditor (stringEditor (const True) True,emptyParams) True "")
     ,   mkField
-            (paraName <<<- ParaName "An URL to load prebuild metadata" $ emptyParams)
+            (paraName <<<- ParaName "URL from which to download prebuilt metadata" $ emptyParams)
             retrieveURL
             (\b a -> a{retrieveURL = b})
-            (stringEditor (\ _ -> True))
+            (stringEditor (\ _ -> True) True)
     ,   mkField
-            (paraName <<<- ParaName "A strategy for downloading prebuild metadata" $ emptyParams)
+            (paraName <<<- ParaName "Strategy for downloading prebuilt metadata" $ emptyParams)
             retrieveStrategy
             (\b a -> a{retrieveStrategy = b})
-            (enumEditor ["Retrieve then build","Build then retrieve","Never retrieve"])
-    ,   mkField
-            (paraName <<<- ParaName "Port number for server connection" $ emptyParams)
-            serverPort
-            (\b a -> a{serverPort = b})
-            (intEditor (1.0, 65535.0, 1.0))
-    ,   mkField
-            (paraName <<<- ParaName "End the server with last connection" $ emptyParams)
-            endWithLastConn
-            (\b a -> a{endWithLastConn = b})
-            boolEditor]
+            (enumEditor ["Try to download and then build locally if that fails","Try to build locally and then download if that fails","Never download (just try to build locally)"])]
 
 --
--- | Called when leksah ist first called (the .leksah-xx directory does not exist)
+-- | Called when leksah is first called (the .leksah-xx directory does not exist)
 --
-firstStart :: Prefs -> IO ()
+firstStart :: Prefs -> IO Bool
 firstStart prefs = do
     dataDir     <- getDataDir
     prefsPath   <- getConfigFilePathForLoad standardPreferencesFilename Nothing dataDir
     prefs       <- readPrefs prefsPath
     configDir   <- getConfigDir
     dialog      <- dialogNew
+    setLeksahIcon dialog
+    set dialog [
+        windowTitle := "Welcome to Leksah, the Haskell IDE",
+        windowWindowPosition := WinPosCenter]
     dialogAddButton dialog "gtk-ok" ResponseOk
     dialogAddButton dialog "gtk-cancel" ResponseCancel
     vb          <- dialogGetUpper dialog
-    label       <- labelNew (Just ("Welcome to Leksah, the Haskell IDE.\n" ++
-        "At the first start, Leksah will collect and download metadata about your installed haskell packages.\n" ++
-        "You can add folders under which you have sources for Haskell packages not available from Hackage.\n" ++
-        "If you are not shure what to do, just keep the defaults \n" ++
-        "This process may take a long time, so be patient."))
+    label       <- labelNew (Just (
+        "Before you start using Leksah it will collect and download metadata about your installed Haskell packages.\n" ++
+        "You can add folders under which you have sources for Haskell packages not available from Hackage."))
     (widget, setInj, getExt,notifier) <- buildEditor (fDescription configDir) prefs
-    boxPackStart vb label PackGrow 7
-    boxPackStart vb widget PackGrow 7
-    widgetSetSizeRequest dialog 800 640
+    boxPackStart vb label PackNatural 7
+    sw <- scrolledWindowNew Nothing Nothing
+    scrolledWindowAddWithViewport sw widget
+    scrolledWindowSetPolicy sw PolicyNever PolicyAutomatic
+    boxPackStart vb sw PackGrow 7
+    windowSetDefaultSize dialog 800 630
     widgetShowAll dialog
     response <- dialogRun dialog
     widgetHide dialog
@@ -338,7 +392,7 @@ firstStart prefs = do
             case mbNewPrefs of
                 Nothing -> do
                     sysMessage Normal "No dialog results"
-                    return ()
+                    return False
                 Just newPrefs -> do
                     fp <- getConfigFilePathForSave standardPreferencesFilename
                     writePrefs fp newPrefs
@@ -351,10 +405,25 @@ firstStart prefs = do
                                        SP.serverPort        = serverPort newPrefs,
                                        SP.endWithLastConn   = endWithLastConn newPrefs})
                     firstBuild newPrefs
-        _ ->     widgetDestroy dialog
+                    return True
+        _ -> do
+            widgetDestroy dialog
+            return False
+
+setLeksahIcon :: (WindowClass self) => self -> IO ()
+setLeksahIcon window = do
+    dataDir <- getDataDir
+    let iconPath = dataDir </> "pics" </> "leksah.png"
+    iconExists  <-  doesFileExist iconPath
+    when iconExists $
+        windowSetIconFromFile window iconPath
 
 firstBuild newPrefs = do
     dialog      <- dialogNew
+    setLeksahIcon dialog
+    set dialog [
+        windowTitle := "Leksah: Updating Metadata",
+        windowWindowPosition := WinPosCenter]
     vb          <- dialogGetUpper dialog
     progressBar <- progressBarNew
     progressBarSetText progressBar "Please wait while Leksah collects information about Haskell packages on your system"
