@@ -1,4 +1,4 @@
-{-# OPTIONS_GHC -XScopedTypeVariables #-}
+{-# LANGUAGE CPP, ScopedTypeVariables #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.LogRef
@@ -50,11 +50,11 @@ import IDE.Pane.SourceBuffer
 import IDE.Pane.Log
 import IDE.Utils.Tool
 import System.FilePath (equalFilePath)
-import System.Directory (canonicalizePath)
 import Data.List (stripPrefix, elemIndex, isPrefixOf)
 import Data.Maybe (catMaybes)
 import System.Exit (ExitCode(..))
 import System.Log.Logger (debugM)
+import IDE.Utils.FileUtils(myCanonicalizePath)
 
 showSourceSpan :: LogRef -> String
 showSourceSpan = displaySrcSpan . logRefSrcSpan
@@ -79,7 +79,8 @@ forOpenLogRefs f = do
     allBufs <- allBuffers
     forM_ [0 .. ((length logRefs)-1)] (\index -> do
         let ref = logRefs !! index
-        fpc <- liftIO $ canonicalizePath $ logRefFullFilePath ref
+            fp = logRefFullFilePath ref
+        fpc <- liftIO $ myCanonicalizePath fp
         forM_ (filter (\buf -> case (fileName buf) of
                 Just fn -> equalFilePath fpc fn
                 Nothing -> False) allBufs) (f index ref))
@@ -244,6 +245,12 @@ lastContext = do
             setCurrentContext (Just new)
             selectRef $ Just new
 
+#if MIN_VERSION_ghc(7,0,1)
+fixColumn c = max 0 (c - 1)
+#else
+fixColumn = id
+#endif
+
 srcSpanParser :: CharParser () SrcSpan
 srcSpanParser = try (do
         filePath <- many (noneOf ":")
@@ -259,7 +266,7 @@ srcSpanParser = try (do
         char ','
         endCol <- int
         char ')'
-        return $ SrcSpan filePath beginLine beginCol endLine endCol)
+        return $ SrcSpan filePath beginLine (fixColumn beginCol) endLine (fixColumn endCol))
     <|> try (do
         filePath <- many (noneOf ":")
         char ':'
@@ -268,14 +275,14 @@ srcSpanParser = try (do
         beginCol <- int
         char '-'
         endCol <- int
-        return $ SrcSpan filePath line beginCol line endCol)
+        return $ SrcSpan filePath line (fixColumn beginCol) line (fixColumn endCol))
     <|> try (do
         filePath <- many (noneOf ":")
         char ':'
         line <- int
         char ':'
         col <- int
-        return $ SrcSpan filePath line col line col)
+        return $ SrcSpan filePath line (fixColumn col) line (fixColumn col))
     <?> "srcLocParser"
 
 data BuildError =   BuildLine
@@ -294,6 +301,7 @@ buildLineParser = try (do
         many (anyChar)
         return BuildLine)
     <|> try (do
+        whiteSpace
         span <- srcSpanParser
         char ':'
         whiteSpace
@@ -387,8 +395,8 @@ logOutput output = do
     logOutputLines defaultLineLogger output
     return ()
 
-logOutputForBuild :: FilePath -> Bool -> [ToolOutput] -> IDEAction
-logOutputForBuild rootPath backgroundBuild output = do
+logOutputForBuild :: IDEPackage -> Bool -> [ToolOutput] -> IDEAction
+logOutputForBuild package backgroundBuild output = do
     ideRef <- ask
     log    <- getLog
     unless backgroundBuild $ liftIO $ bringPaneToFront log
@@ -428,7 +436,7 @@ logOutputForBuild rootPath backgroundBuild output = do
                         sysMessage Normal (show e)
                         readAndShow remainingOutput ideR log False errs
                     (Right ne@(ErrorLine span refType str),_) ->
-                        readAndShow remainingOutput ideR log True ((LogRef span rootPath str (lineNr,lineNr) refType):errs)
+                        readAndShow remainingOutput ideR log True ((LogRef span package str (lineNr,lineNr) refType):errs)
                     (Right (OtherLine str1),(LogRef span rootPath str (l1,l2) refType):tl) ->
                         if inError
                             then readAndShow remainingOutput ideR log True ((LogRef span
@@ -471,38 +479,38 @@ logOutputForBuild rootPath backgroundBuild output = do
                                             ++ show warnNum ++ " warnings -----\n") FrameTag
                 return errs
 
-logOutputForBreakpoints :: FilePath -> [ToolOutput] -> IDEAction
-logOutputForBreakpoints rootPath output = do
+logOutputForBreakpoints :: IDEPackage -> [ToolOutput] -> IDEAction
+logOutputForBreakpoints package output = do
     breaks <- logOutputLines (\log out -> do
         case out of
             ToolOutput line -> do
                 logLineNumber <- liftIO $ appendLog log (line ++ "\n") LogTag
                 case parse breaksLineParser "" line of
                     Right (BreakpointDescription n span) ->
-                        return $ Just $ LogRef span rootPath line (logLineNumber, logLineNumber) BreakpointRef
+                        return $ Just $ LogRef span package line (logLineNumber, logLineNumber) BreakpointRef
                     _ -> return Nothing
             _ -> do
                 defaultLineLogger log out
                 return Nothing) output
     setBreakpointList $ catMaybes breaks
 
-logOutputForSetBreakpoint :: FilePath -> [ToolOutput] -> IDEAction
-logOutputForSetBreakpoint rootPath output = do
+logOutputForSetBreakpoint :: IDEPackage -> [ToolOutput] -> IDEAction
+logOutputForSetBreakpoint package output = do
     breaks <- logOutputLines (\log out -> do
         case out of
             ToolOutput line -> do
                 logLineNumber <- liftIO $ appendLog log (line ++ "\n") LogTag
                 case parse setBreakpointLineParser "" line of
                     Right (BreakpointDescription n span) ->
-                        return $ Just $ LogRef span rootPath line (logLineNumber, logLineNumber) BreakpointRef
+                        return $ Just $ LogRef span package line (logLineNumber, logLineNumber) BreakpointRef
                     _ -> return Nothing
             _ -> do
                 defaultLineLogger log out
                 return Nothing) output
     addLogRefs $ catMaybes breaks
 
-logOutputForContext :: FilePath -> (String -> [SrcSpan]) -> [ToolOutput] -> IDEAction
-logOutputForContext rootPath getContexts output = do
+logOutputForContext :: IDEPackage -> (String -> [SrcSpan]) -> [ToolOutput] -> IDEAction
+logOutputForContext package getContexts output = do
     refs <- fmap catMaybes $ logOutputLines (\log out -> do
         case out of
             ToolOutput line -> do
@@ -510,7 +518,7 @@ logOutputForContext rootPath getContexts output = do
                 let contexts = getContexts line
                 if null contexts
                     then return Nothing
-                    else return $ Just $ LogRef (last contexts) rootPath line (logLineNumber, logLineNumber) ContextRef
+                    else return $ Just $ LogRef (last contexts) package line (logLineNumber, logLineNumber) ContextRef
             _ -> do
                 defaultLineLogger log out
                 return Nothing) output
@@ -518,8 +526,8 @@ logOutputForContext rootPath getContexts output = do
         addLogRefs [last refs]
         lastContext
 
-logOutputForLiveContext :: FilePath -> [ToolOutput] -> IDEAction
-logOutputForLiveContext rootPath = logOutputForContext rootPath getContexts
+logOutputForLiveContext :: IDEPackage -> [ToolOutput] -> IDEAction
+logOutputForLiveContext package = logOutputForContext package getContexts
     where
         getContexts [] = []
         getContexts line@(x:xs) = case stripPrefix "Stopped at " line of
@@ -528,8 +536,8 @@ logOutputForLiveContext rootPath = logOutputForContext rootPath getContexts
                 _          -> getContexts xs
             _ -> getContexts xs
 
-logOutputForHistoricContext :: FilePath -> [ToolOutput] -> IDEAction
-logOutputForHistoricContext rootPath = logOutputForContext rootPath getContexts
+logOutputForHistoricContext :: IDEPackage -> [ToolOutput] -> IDEAction
+logOutputForHistoricContext package = logOutputForContext package getContexts
     where
         getContexts line = case stripPrefix "Logged breakpoint at " line of
             Just rest -> case parse srcSpanParser "" rest of
