@@ -35,7 +35,8 @@ module IDE.Metainfo.Provider (
 ,   getPackageImportInfo -- Scope for the import tool
 ) where
 
-import System.IO
+import System.IO (hClose, openBinaryFile, IOMode(..))
+import System.IO.Strict (readFile)
 import qualified Data.Map as Map
 import Control.Monad
 import Control.Monad.Trans
@@ -64,7 +65,7 @@ import Distribution.Text (display)
 import IDE.Core.Serializable ()
 import Data.Map (Map(..))
 import Control.Exception (SomeException(..), catch)
-import Prelude hiding(catch)
+import Prelude hiding(catch, readFile)
 import IDE.Utils.ServerConnection(doServerCommand)
 
 trace a b = b
@@ -81,7 +82,7 @@ initInfo continuation = do
     prefs  <- readIDE prefs
     if collectAtStart prefs
         then do
-            ideMessage Normal "Now updating sytem metadata ..."
+            ideMessage Normal "Now updating system metadata ..."
             callCollector False True True $ \ _ -> do
                 ideMessage Normal "Now loading metadata ..."
                 loadSystemInfo
@@ -383,7 +384,7 @@ loadInfosForPackage dirPath pid = do
             file            <-  openBinaryFile filePath ReadMode
             trace ("now loading metadata for package " ++ packageIdentifierToString pid) return ()
             bs              <-  BSL.hGetContents file
-            let (metadataVersion', packageInfo) =   decodeSer bs
+            let (metadataVersion'::Integer, packageInfo::PackageDescr) = decodeSer bs
             if metadataVersion /= metadataVersion'
                 then do
                     hClose file
@@ -430,7 +431,7 @@ loadInfosForModule filePath  = do
         then catch (do
             file            <-  openBinaryFile filePath ReadMode
             bs              <-  BSL.hGetContents file
-            let (metadataVersion', moduleInfo) =   decodeSer bs
+            let (metadataVersion'::Integer, moduleInfo::ModuleDescr) = decodeSer bs
             if metadataVersion /= metadataVersion'
                 then do
                     hClose file
