@@ -23,6 +23,7 @@ module IDE.Pane.Modules (
 ,   reloadKeepSelection
 ,   replaySelHistory
 ,   replayScopeHistory
+,   addModule
 ) where
 
 import Graphics.UI.Gtk hiding (get)
@@ -41,25 +42,24 @@ import Data.IORef
 import IDE.Core.State
 import IDE.Pane.Info
 import IDE.Pane.SourceBuffer
-import Control.Event hiding (Event)
 import Distribution.ModuleName
 import Distribution.Text (simpleParse,display)
 import Data.Typeable (Typeable(..))
 import Control.Exception (SomeException(..),catch)
-import IDE.Package (packageConfig,addModuleToPackageDescr,getModuleTemplate,getPackageDescriptionAndPath)
+import IDE.Package (packageConfig,addModuleToPackageDescr,delModuleFromPackageDescr,getEmptyModuleTemplate,getPackageDescriptionAndPath)
 import Distribution.PackageDescription (allBuildInfo,hsSourceDirs)
-import System.FilePath ((</>),dropFileName)
-import System.Directory (doesFileExist,createDirectoryIfMissing)
+import System.FilePath (takeBaseName, (</>),dropFileName)
+import System.Directory (doesFileExist,createDirectoryIfMissing, removeFile)
 import Graphics.UI.Editor.MakeEditor (buildEditor,FieldDescription(..),mkField)
 import Graphics.UI.Editor.Parameters (paraMultiSel,Parameter(..),emptyParams,(<<<-),paraName)
-import Graphics.UI.Editor.Simple (boolEditor,okCancelFields,staticListEditor,stringEditor)
-import Graphics.UI.Editor.Basics (eventPaneName,GUIEventSelector(..))
+import Graphics.UI.Editor.Simple (boolEditor,staticListEditor,stringEditor)
 import qualified System.IO.UTF8 as UTF8  (writeFile)
 import IDE.Utils.GUIUtils (stockIdFromType)
 import IDE.Metainfo.Provider
        (getSystemInfo, getWorkspaceInfo, getPackageInfo)
 import System.Log.Logger (infoM)
 import Default (Default(..))
+import IDE.Workspaces (packageTry_)
 
 -- | A modules pane description
 --
@@ -695,42 +695,48 @@ type ModTree = Tree (String, Maybe (ModuleDescr,PackageDescr))
 --
 buildModulesTree :: (SymbolTable alpha, SymbolTable beta) =>  (PackScope alpha,PackScope beta ) -> ModTree
 buildModulesTree (PackScope localMap _,PackScope otherMap _) =
-    let flatPairs           =   concatMap (\p -> map (\m -> (m,p)) (pdModules p))
+    let modDescrPackDescr =   concatMap (\p -> map (\m -> (m,p)) (pdModules p))
                                     (Map.elems localMap ++ Map.elems otherMap)
-        resultTree          =   foldl' insertPairsInTree defaultRoot flatPairs
+        resultTree        =   foldl' insertPairsInTree defaultRoot modDescrPackDescr
         in sortTree resultTree
-    where
-    insertPairsInTree :: ModTree -> (ModuleDescr,PackageDescr) -> ModTree
-    insertPairsInTree tree pair =
-        let nameArray           =   components $ modu $ mdModuleId $ fst pair
-            (startArray,last)   =   splitAt (length nameArray - 1) nameArray
-            pairedWith          =   (map (\n -> (n,Nothing)) startArray) ++ [(head last,Just pair)]
-        in  insertNodesInTree pairedWith tree
 
-    insertNodesInTree :: [(String,Maybe (ModuleDescr,PackageDescr))] -> ModTree -> ModTree
-    insertNodesInTree list@[(str2,Just pair)] (Node (str1,pairs) forest) =
-        (Node (str1,pairs) (makeNodes list : forest))
+insertPairsInTree :: ModTree -> (ModuleDescr,PackageDescr) -> ModTree
+insertPairsInTree tree pair =
+    let nameArray           =   components $ modu $ mdModuleId $ fst pair
+        (startArray,last)   =   splitAt (length nameArray - 1) nameArray
+        pairedWith          =   (map (\n -> (n,Nothing)) startArray) ++ [(head last,Just pair)]
+    in  insertNodesInTree pairedWith tree
 
-    insertNodesInTree  list@((str2,pair):tl) (Node (str1,pairs) forest) =
-        case partition (\ (Node (s,_) _) -> s == str2) forest of
-            ([],_)              ->  (Node (str1,pairs)  (makeNodes list : forest))
-            ([found],rest)  ->  (Node (str1,pairs)  (insertNodesInTree tl found : rest))
-            (foundList,rest)  -> (Node (str1,pairs)  (insertNodesInTree tl (head foundList) : rest))
-                -- TODO make smart
-    insertNodesInTree [] t      =   t
 
-    makeNodes :: [(String,Maybe (ModuleDescr,PackageDescr))] -> ModTree
-    makeNodes [(str,mbPair)]    =   Node (str,mbPair) []
-    makeNodes ((str,mbPair):tl) =   Node (str,mbPair) [makeNodes tl]
-    makeNodes _                 =   throwIDE "Impossible in makeNodes"
+insertNodesInTree :: [(String, Maybe (ModuleDescr,PackageDescr))] -> ModTree -> ModTree
+insertNodesInTree  [p1@(str1,Just pair)] (Node p2@(str2,mbPair) forest2) =
+    case partition (\ (Node (s,_) _) -> s == str1) forest2 of
+        ([found],rest) -> case found of
+                            Node p3@(_,Nothing) forest3 ->
+                                Node p2 (Node p1 forest3 : rest)
+                            Node p3@(_,Just pair3) forest3 ->
+                                Node p2 (Node p1 [] : Node p3 forest3 : rest)
+        ([],rest)      -> Node p2 (Node p1 [] : forest2)
+        (found,rest)   -> case head found of
+                            Node p3@(_,Nothing) forest3 ->
+                                Node p2 (Node p1 forest3 : tail found ++ rest)
+                            Node p3@(_,Just pair3) forest3 ->
+                                Node p2 (Node p1 [] : Node p3 forest3 : tail found ++ rest)
 
-breakAtDots :: [String] -> String -> [String]
-breakAtDots res []          =   reverse res
-breakAtDots res toBreak     =   let (newRes,newToBreak) = span (\c -> c /= '.') toBreak
-                                in  if null newToBreak
-                                        then reverse (newRes : res)
-                                        else breakAtDots (newRes : res) (tail newToBreak)
+insertNodesInTree li@(hd@(str1,Nothing):tl) (Node p@(str2,mbPair) forest) =
+    case partition (\ (Node (s,_) _) -> s == str1) forest of
+        ([found],rest) -> Node p  (insertNodesInTree tl found : rest)
+        ([],rest)      -> Node p  (makeNodes li : forest)
+        (found,rest)   -> Node p  (insertNodesInTree tl (head found) : tail found ++ rest)
 
+insertNodesInTree [] n = n
+insertNodesInTree _ _      =   error "Modules>>insertNodesInTree: Should not happen2"
+
+
+makeNodes :: [(String,Maybe (ModuleDescr,PackageDescr))] -> ModTree
+makeNodes [(str,mbPair)]    =   Node (str,mbPair) []
+makeNodes ((str,mbPair):tl) =   Node (str,mbPair) [makeNodes tl]
+makeNodes _                 =   throwIDE "Impossible in makeNodes"
 
 instance Ord a => Ord (Tree a) where
     compare (Node l1 _) (Node l2 _) =  compare l1 l2
@@ -768,14 +774,45 @@ treeViewPopup ideR  store treeView (Button _ click _ _ _ _ button _ _) = do
             item5 `onActivateLeaf` (treeViewCollapseAll treeView)
             sep2 <- separatorMenuItemNew
             item6           <-  menuItemNewWithLabel "Add module"
-            item6 `onActivateLeaf` (reflectIDE (addModule treeView store) ideR)
+            item6 `onActivateLeaf` (reflectIDE (packageTry_ $ addModule' treeView store) ideR)
+            item7           <-  menuItemNewWithLabel "Delete module"
+            item7 `onActivateLeaf` do
+                sel         <-  getSelectionTree treeView store
+                case sel of
+                    Just (_,Just (m,_)) -> case mdMbSourcePath m of
+                                            Nothing     ->  return ()
+                                            Just fp     ->  do
+                                                resp <- reflectIDE (respDelModDialog)ideR
+                                                if (resp == False) then return ()
+                                                    else do
+                                                        exists <- doesFileExist fp
+                                                        if exists
+                                                           then do
+                                                             reflectIDE (liftIO $ removeFile fp) ideR
+                                                             reflectIDE (packageTry_ $ delModule treeView store)ideR
+                                                           else do
+                                                             reflectIDE (packageTry_ $ delModule treeView store)ideR
+                                                        reflectIDE (packageTry_ packageConfig) ideR
+                                                        return ()
+                    otherwise       ->  return ()
+            sel         <-  getSelectionTree treeView store
+            case sel of
+                Just (s, Nothing) -> do
+                    mapM_ (menuShellAppend theMenu) [castToMenuItem item1, castToMenuItem sep1, castToMenuItem item2,
+                        castToMenuItem item3, castToMenuItem item4, castToMenuItem item5, castToMenuItem sep2,
+                        castToMenuItem item6]
+                    menuPopup theMenu Nothing
+                    widgetShowAll theMenu
+                    return True
+                Just (_,Just (m,_)) -> do
+                    mapM_ (menuShellAppend theMenu) [castToMenuItem item1, castToMenuItem sep1, castToMenuItem item2,
+                        castToMenuItem item3, castToMenuItem item4, castToMenuItem item5, castToMenuItem sep2,
+                        castToMenuItem item6, castToMenuItem item7]
+                    menuPopup theMenu Nothing
+                    widgetShowAll theMenu
+                    return True
+                otherwise -> return True
 
-            mapM_ (menuShellAppend theMenu) [castToMenuItem item1, castToMenuItem sep1, castToMenuItem item2,
-                castToMenuItem item3, castToMenuItem item4, castToMenuItem item5, castToMenuItem sep2,
-                castToMenuItem item6{--,item7,item8--}]
-            menuPopup theMenu Nothing
-            widgetShowAll theMenu
-            return True
         else if button == LeftButton && click == DoubleClick
                 then do sel         <-  getSelectionTree treeView store
                         case sel of
@@ -787,6 +824,7 @@ treeViewPopup ideR  store treeView (Button _ click _ _ _ _ button _ _) = do
                             otherwise       ->  return ()
                         return True
                 else return False
+
 treeViewPopup _ _ _ _ = throwIDE "treeViewPopup wrong event type"
 
 descrViewPopup :: IDERef
@@ -989,41 +1027,80 @@ collapseHere treeView = do
         []     -> return ()
         (hd:_) -> treeViewCollapseRow treeView hd >> return ()
 
-addModule :: TreeView -> TreeStore (String, Maybe (ModuleDescr,PackageDescr)) -> IDEAction
-addModule treeView store = do
+delModule :: TreeView -> TreeStore (String, Maybe (ModuleDescr,PackageDescr)) -> PackageAction
+delModule treeview store = do
+    window <- lift $ getMainWindow
+    sel   <- liftIO $ treeViewGetSelection treeview
+    paths <- liftIO $ treeSelectionGetSelectedRows sel
+    categories <- case paths of
+        []     -> return []
+        (treePath:_) -> liftIO $ mapM (treeStoreGetValue store)
+                                    $ map (\n -> take n treePath)  [1 .. length treePath]
+
+    lift $ ideMessage Normal ("categories: " ++ (show categories))
+
+    let modPacDescr = snd(last categories)
+    case modPacDescr of
+        Nothing     ->   lift $ ideMessage Normal "This should never be shown!"
+        Just(md,_)  -> do
+                         let modName = modu.mdModuleId $ md
+                         lift $ ideMessage Normal ("modName: " ++ (show modName))
+                         delModuleFromPackageDescr modName
+
+respDelModDialog :: IDEM (Bool)
+respDelModDialog = do
+    window <- getMainWindow
+    resp <- liftIO $ do
+        dia <- messageDialogNew (Just window) [] MessageQuestion ButtonsCancel "Are you sure?"
+        dialogAddButton dia "_Delete Module" (ResponseUser 1)
+        dialogSetDefaultResponse dia ResponseCancel
+        set dia [ windowWindowPosition := WinPosCenterOnParent ]
+        resp <- dialogRun dia
+        widgetDestroy dia
+        return resp
+    return $ resp == ResponseUser 1
+
+addModule' :: TreeView -> TreeStore (String, Maybe (ModuleDescr,PackageDescr)) -> PackageAction
+addModule' treeView store = do
     sel   <- liftIO $ treeViewGetSelection treeView
     paths <- liftIO $ treeSelectionGetSelectedRows sel
     categories <- case paths of
         []     -> return []
         (treePath:_) -> liftIO $ mapM (treeStoreGetValue store)
                                     $ map (\n -> take n treePath)  [1 .. length treePath]
-    mbPD <- getPackageDescriptionAndPath
+    addModule categories
+
+addModule categories = do
+    mbPD <- lift $ getPackageDescriptionAndPath
     case mbPD of
-        Nothing             -> ideMessage Normal "No package description"
+        Nothing             -> lift $ ideMessage Normal "No package description"
         Just (pd,cabalPath) -> let srcPaths = nub $ concatMap hsSourceDirs $ allBuildInfo pd
                                    rootPath = dropFileName cabalPath
                                    modPath  = foldr (\a b -> a ++ "." ++ b) ""
                                                 (map fst categories)
                                in do
-            window' <- getMainWindow
+            window' <- lift getMainWindow
             mbResp <- liftIO $ addModuleDialog window' modPath srcPaths
             case mbResp of
                 Nothing                -> return ()
                 Just (AddModule modPath srcPath isExposed) ->
                     case simpleParse modPath of
-                        Nothing         -> ideMessage Normal ("Not a valid module name :" ++ modPath)
+                        Nothing         -> lift $ ideMessage Normal ("Not a valid module name :" ++ modPath)
                         Just moduleName -> do
                             let  target = srcPath </> toFilePath moduleName ++ ".hs"
                             liftIO $ createDirectoryIfMissing True (dropFileName target)
                             alreadyExists <- liftIO $ doesFileExist target
                             if alreadyExists
-                                then ideMessage Normal "File already exists"
+                                then do
+                                    lift $ ideMessage Normal ("File already exists! Importing existing file " ++ takeBaseName target ++ ".hs")
+                                    addModuleToPackageDescr moduleName isExposed
+                                    packageConfig
                                 else do
-                                    template <- liftIO $ getModuleTemplate pd modPath
+                                    template <- liftIO $ getEmptyModuleTemplate pd modPath
                                     liftIO $ UTF8.writeFile target template
                                     addModuleToPackageDescr moduleName isExposed
                                     packageConfig
-                                    fileOpenThis target
+                                    lift $ fileOpenThis target
 
 
 --  Yet another stupid little dialog
@@ -1039,17 +1116,20 @@ addModuleDialog parent modString sourceRoots = do
     lower                      <-   dialogGetActionArea dia
     (widget,inj,ext,_)         <-   buildEditor (moduleFields sourceRoots)
                                         (AddModule modString (head sourceRoots) False)
-    (widget2,_,_,notifier)     <-   buildEditor okCancelFields ()
-    registerEvent notifier Clicked (Left (\e -> do
-            case eventPaneName e of
-                "Ok"    ->  dialogResponse dia ResponseOk
-                _       ->  dialogResponse dia ResponseCancel
-            return e))
+    bb      <-  hButtonBoxNew
+    closeB  <-  buttonNewFromStock "gtk-cancel"
+    save    <-  buttonNewFromStock "gtk-ok"
+    boxPackEnd bb closeB PackNatural 0
+    boxPackEnd bb save PackNatural 0
+    save `onClicked` (dialogResponse dia ResponseOk)
+    closeB `onClicked` (dialogResponse dia ResponseCancel)
     boxPackStart upper widget PackGrow 7
-    boxPackStart lower widget2 PackNatural 7
+    boxPackStart lower bb PackNatural 7
+    set save [widgetCanDefault := True]
+    widgetGrabDefault save
     widgetShowAll dia
-    resp <- dialogRun dia
-    value                      <- ext (AddModule modString (head sourceRoots) False)
+    resp  <- dialogRun dia
+    value <- ext (AddModule modString (head sourceRoots) False)
     widgetDestroy dia
     --find
     case resp of
@@ -1063,7 +1143,7 @@ moduleFields list = VFD emptyParams [
                     $ emptyParams)
             moduleName
             (\ a b -> b{moduleName = a})
-            (stringEditor (\s -> True)),
+            (stringEditor (const True) True),
         mkField
             (paraName <<<- ParaName ("Root of the source path")
                 $ paraMultiSel <<<- ParaMultiSel False
