@@ -22,6 +22,7 @@ module IDE.Pane.PackageEditor (
 ,   choosePackageFile
 
 ,   hasConfigs
+,   standardSetup
 ) where
 
 import Graphics.UI.Gtk
@@ -78,7 +79,8 @@ import Graphics.UI.Editor.Simple
      comboSelectionEditor,
      multilineStringEditor,
      stringEditor)
-import Graphics.UI.Editor.Basics (Editor(..))
+import Graphics.UI.Editor.Basics
+       (Notifier, Editor(..), GUIEventSelector(..), GUIEvent(..))
 import Distribution.Compiler
     (CompilerFlavor(..))
 import Distribution.Simple
@@ -87,38 +89,42 @@ import Distribution.Simple
      VersionRange(..))
 import Default (Default(..))
 import IDE.Utils.GUIUtils
+import IDE.Pane.SourceBuffer (fileOpenThis)
+import Control.Event (EventSource(..))
 #if MIN_VERSION_Cabal(1,8,0)
 #else
 import Distribution.License
 #endif
 
+import qualified Graphics.UI.Gtk.Gdk.Events as GTK (Event(..))
+import Data.List (sort)
+
+
 -- ---------------------------------------------------------------------
 -- The exported stuff goes here
 --
 
 choosePackageDir :: Window -> Maybe FilePath -> IO (Maybe FilePath)
-choosePackageDir window mbDir = chooseDir window "Select root folder for project" mbDir
+choosePackageDir window mbDir = chooseDir window "Select root folder for package" mbDir
 
 choosePackageFile :: Window -> Maybe FilePath -> IO (Maybe FilePath)
 choosePackageFile window mbDir = chooseFile window "Select cabal package file (.cabal)" mbDir
 
-packageEdit :: IDEAction
+packageEdit :: PackageAction
 packageEdit = do
-    mbActivePackage <- readIDE activePack
-    case mbActivePackage of
-        Nothing -> sysMessage Normal "No active package to edit"
-        Just idePackage -> do
-            let dirName = dropFileName (ipdCabalFile idePackage)
-            modules <- liftIO $ allModules dirName
-            package <- liftIO $ readPackageDescription normal (ipdCabalFile idePackage)
-            if hasConfigs package
-                then do
-                    ideMessage High ("Cabal file with configurations can't be edited with the "
-                        ++ "current version of the editor")
-                    return ()
-                else do
-                    editPackage (flattenPackageDescription package) dirName  modules (\ _ -> return ())
-                    return ()
+    idePackage <- ask
+    let dirName = dropFileName (ipdCabalFile idePackage)
+    modules <- liftIO $ allModules dirName
+    package <- liftIO $ readPackageDescription normal (ipdCabalFile idePackage)
+    if hasConfigs package
+        then do
+            lift $ ideMessage High ("Cabal file with configurations can't be edited with the "
+                ++ "current version of the editor")
+            lift $ fileOpenThis $ ipdCabalFile idePackage
+            return ()
+        else do
+            lift $ editPackage (flattenPackageDescription package) dirName  modules (\ _ -> return ())
+            return ()
 
 hasConfigs :: GenericPackageDescription -> Bool
 hasConfigs gpd =
@@ -133,27 +139,29 @@ hasConfigs gpd =
     in libConds || exeConds
 
 
-packageNew' :: Maybe FilePath -> (FilePath -> IDEAction) -> IDEAction
+packageNew' :: Maybe FilePath -> (Bool -> FilePath -> IDEAction) -> IDEAction
 packageNew' mbDir activateAction = do
     windows  <- getWindows
     mbDirName <- liftIO $ choosePackageDir (head windows) mbDir
     case mbDirName of
         Nothing -> return ()
         Just dirName -> do
-            cfn <-  liftIO $ cabalFileName dirName
-            continue <- do
-                if isJust cfn
-                    then liftIO $ do
-                        md <- messageDialogNew Nothing [] MessageQuestion ButtonsYesNo
-                                    $ "There is already a .cabal file in this directory."
-                                    ++  " Continue anyway?"
+            mbCabalFile <-  liftIO $ cabalFileName dirName
+            case mbCabalFile of
+                Just cfn -> do
+                    window <- getMainWindow
+                    add <- liftIO $ do
+                        md <- messageDialogNew (Just window) [] MessageQuestion ButtonsCancel
+                            $ "There is already file " ++ takeFileName cfn ++ " in this directory. "
+                            ++ "Would you like to add this package to the workspace?"
+                        dialogAddButton md "_Add Package" (ResponseUser 1)
+                        dialogSetDefaultResponse md (ResponseUser 1)
+                        set md [ windowWindowPosition := WinPosCenterOnParent ]
                         rid <- dialogRun md
                         widgetDestroy md
-                        case rid of
-                            ResponseYes ->  return True
-                            otherwise   ->  return False
-                    else return True
-            when continue $ do
+                        return $ rid == ResponseUser 1
+                    when add $ activateAction False cfn
+                Nothing -> do
                     modules <- liftIO $ do
                         b1 <- doesFileExist (dirName </> "Setup.hs")
                         b2 <- doesFileExist (dirName </> "Setup.lhs")
@@ -173,7 +181,7 @@ packageNew' mbDir activateAction = do
                             exeName    = (takeBaseName dirName),
                             modulePath = "Main.hs",
                             buildInfo  = emptyBuildInfo {hsSourceDirs = ["src"]}}]
-                        } dirName modules activateAction
+                        } dirName modules (activateAction True)
                     return ()
 
 standardSetup = "#!/usr/bin/runhaskell \n"
@@ -192,6 +200,13 @@ data PackageDescriptionEd = PDE {
     exes         :: [Executable'],
     mbLib        :: Maybe Library',
     bis          :: [BuildInfo]}
+        deriving Eq
+
+comparePDE a b = do
+    when (pd a /= pd b) $ putStrLn  "pd"
+    when (exes a /= exes b) $ putStrLn  "exes"
+    when (mbLib a /= mbLib b) $ putStrLn  "mbLib"
+    when (bis a /= bis b) $ putStrLn  "bis"
 
 fromEditor :: PackageDescriptionEd -> PackageDescription
 fromEditor (PDE pd exes' mbLib' buildInfos) =
@@ -211,7 +226,7 @@ toEditor pd =
                             (zip (executables pd) [0 .. length (executables pd)])
             (mbLib,bis2) = case library pd of
                     Nothing                -> (Nothing,bis)
-                    Just (Library mn b bi) -> (Just (Library' mn b (length bis)), bis ++ [bi])
+                    Just (Library mn b bi) -> (Just (Library' (sort mn) b (length bis)), bis ++ [bi])
             bis3 = if null bis2
                         then [emptyBuildInfo]
                         else bis2
@@ -222,7 +237,8 @@ toEditor pd =
 --
 
 data PackagePane             =   PackagePane {
-    packageBox              ::   VBox
+    packageBox              ::   VBox,
+    packageNotifer          ::   Notifier
 } deriving Typeable
 
 
@@ -259,7 +275,7 @@ editPackage packageD packagePath modules afterSaveAction = do
                     (length (bis packageEd))
                     (concatMap (buildInfoD (Just (takeDirectory packagePath)) modules)
                         [0..length (bis packageEd) - 1]))
-                pp nb modules afterSaveAction
+                pp nb modules afterSaveAction packageEd
         Just p -> liftIO $ bringPaneToFront p
 
 initPackage :: FilePath
@@ -269,15 +285,24 @@ initPackage :: FilePath
     -> Notebook
     -> [ModuleName]
     -> (FilePath -> IDEAction)
+    -> PackageDescriptionEd
     -> IDEM ()
-initPackage packageDir packageD packageDescr panePath nb modules afterSaveAction = do
+initPackage packageDir packageD packageDescr panePath nb modules afterSaveAction origPackageD = do
     let fields =  flattenFieldDescription packageDescr
     let initialPackagePath = packageDir </> (display . pkgName . package . pd) packageD ++ ".cabal"
     packageInfos <- liftIO $ getInstalledPackageIds
-    buildThisPane panePath nb
+    mbP <- buildThisPane panePath nb
         (builder' packageDir packageD packageDescr afterSaveAction
-            initialPackagePath modules packageInfos fields)
-    return ()
+            initialPackagePath modules packageInfos fields origPackageD)
+    case mbP of
+        Nothing -> return ()
+        Just (PackagePane{packageNotifer = pn}) -> do
+            liftIO $ triggerEvent pn (GUIEvent {
+                    selector = MayHaveChanged,
+                    gtkEvent = GTK.Event True,
+                    eventText = "",
+                    gtkReturn = True})
+            return ()
 
 builder' :: FilePath ->
     PackageDescriptionEd ->
@@ -287,97 +312,39 @@ builder' :: FilePath ->
     [ModuleName] ->
     [PackageId] ->
     [FieldDescription PackageDescriptionEd] ->
+    PackageDescriptionEd ->
     PanePath ->
     Notebook ->
     Window ->
     IDEM (Maybe PackagePane,Connections)
 builder' packageDir packageD packageDescr afterSaveAction initialPackagePath modules packageInfos fields
-    panePath nb window = reifyIDE $ \ ideR -> do
+    origPackageD panePath nb window  = reifyIDE $ \ ideR -> do
     vb      <-  vBoxNew False 0
-    let packagePane = PackagePane vb
     bb      <-  hButtonBoxNew
-    restore <- buttonNewFromStock "Revert"
     save    <- buttonNewFromStock "gtk-save"
+    widgetSetSensitive save False
     closeB  <- buttonNewFromStock "gtk-close"
     addB    <- buttonNewFromStock "Add Build Info"
     removeB <- buttonNewFromStock "Remove Build Info"
+    label   <-  labelNew Nothing
     boxPackStart bb addB PackNatural 0
     boxPackStart bb removeB PackNatural 0
-    boxPackStart bb restore PackNatural 0
-    boxPackStart bb save PackNatural 0
-    boxPackStart bb closeB PackNatural 0
+    boxPackEnd bb closeB PackNatural 0
+    boxPackEnd bb save PackNatural 0
     (widget, setInj, getExt, notifier)  <-  buildEditor packageDescr packageD
+    let packagePane = PackagePane vb notifier
+    boxPackStart vb widget PackGrow 7
+    boxPackStart vb label PackNatural 0
+    boxPackEnd vb bb PackNatural 7
+
     let fieldNames = map (\fd -> case getParameterPrim paraName (parameters fd) of
                                     Just s -> s
                                     Nothing -> "Unnamed") fields
-    save `onClicked` (do
-        mbNewPackage' <- extractAndValidate packageD [getExt] fieldNames
-        case mbNewPackage' of
-            Nothing -> return ()
-            Just newPackage' -> let newPackage = fromEditor newPackage' in do
-                let packagePath = packageDir </> (display . pkgName . package . pd) newPackage'
-                                                ++ ".cabal"
-                writePackageDescription packagePath newPackage
-                reflectIDE (afterSaveAction packagePath) ideR)
-    closeB `onClicked` (do
-        mbNewPackage' <- extractAndValidate packageD [getExt] fieldNames
-        case mbNewPackage' of
-            Nothing -> do
-                md <- messageDialogNew Nothing [] MessageQuestion ButtonsYesNo
-                                   "Package does not validate. Close anyway?"
-                rid <- dialogRun md
-                widgetDestroy md
-                case rid of
-                    ResponseYes ->  (reflectIDE (closePane packagePane >> return ()) ideR)
-                    otherwise   ->  return ()
-            Just newPackage -> do
-                let packagePath = packageDir </> (display . pkgName . package . pd) newPackage
-                modified <- do
-                    exists <- liftIO $ doesFileExist packagePath
-                    if exists
-                        then do
-                            packageNow    <- readPackageDescription normal packagePath
-                            let simpleNow = flattenPackageDescription packageNow
-                            case mbNewPackage' of
-                                Nothing -> return True --doesn't validate, so something has changed
-                                Just pd -> return (fromEditor pd /=
-                                     simpleNow{buildDepends = reverse (buildDepends simpleNow)})
-                        else return False
-                cancel <- if modified
-                    then do
-                        md <- messageDialogNew (Just window) []
-                                                    MessageQuestion
-                                                    ButtonsNone
-                                                    ("Save changes to Cabal file: "
-                                                        ++ packagePath
-                                                        ++ "?")
-                        dialogAddButton md "_Save" ResponseYes
-                        dialogAddButton md "_Don't Save" ResponseNo
-                        dialogAddButton md "_Cancel" ResponseCancel
-                        resp <- dialogRun md
-                        widgetDestroy md
-                        case resp of
-                            ResponseYes ->   do
-                                case mbNewPackage' of
-                                    Nothing -> return False
-                                    Just newPackage' -> let newPackage = fromEditor newPackage' in do
-                                        let PackageIdentifier (PackageName n) v = package newPackage
-                                        writePackageDescription packagePath newPackage
-                                        return False
-                            ResponseCancel  ->   return True
-                            ResponseNo      ->   return False
-                            _               ->   return False
-                    else return False
-                when (not cancel) (reflectIDE (closePane packagePane >> return ()) ideR))
-    restore `onClicked` (do
-        package <- readPackageDescription normal initialPackagePath
-        setInj (toEditor (flattenPackageDescription package)))
     addB `onClicked` (do
-        mbNewPackage' <- extractAndValidate packageD [getExt] fieldNames
+        mbNewPackage' <- extract packageD [getExt]
         case mbNewPackage' of
             Nothing -> sysMessage Normal "Content doesn't validate"
-            Just pde -> do
-                reflectIDE (do
+            Just pde -> reflectIDE (do
                     closePane packagePane
                     initPackage packageDir pde {bis = bis pde ++ [bis pde !! 0]}
                         (packageDD
@@ -387,25 +354,68 @@ builder' packageDir packageD packageDescr afterSaveAction initialPackagePath mod
                             (length (bis pde) + 1)
                             (concatMap (buildInfoD (Just packageDir) modules)
                                 [0..length (bis pde)]))
-                        panePath nb modules afterSaveAction) ideR)
+                        panePath nb modules afterSaveAction origPackageD) ideR)
     removeB `onClicked` (do
-        mbNewPackage' <- extractAndValidate packageD [getExt] fieldNames
+        mbNewPackage' <- extract packageD [getExt]
         case mbNewPackage' of
             Nothing -> sysMessage Normal "Content doesn't validate"
             Just pde | length (bis pde) == 1  -> sysMessage Normal "Just one Build Info"
                      | otherwise -> reflectIDE (do
+                        closePane packagePane
+                        initPackage packageDir pde{bis = take (length (bis pde) - 1) (bis pde)}
+                            (packageDD
+                                packageInfos
+                                packageDir
+                                modules
+                                (length (bis pde) - 1)
+                                (concatMap (buildInfoD (Just packageDir) modules)
+                                    [0..length (bis pde) - 2]))
+                            panePath nb modules afterSaveAction origPackageD) ideR)
+    closeB `onClicked` (do
+        mbP <- extract packageD [getExt]
+        let hasChanged = case mbP of
+                                Nothing -> False
+                                Just p -> p /= origPackageD
+        if not hasChanged
+            then reflectIDE (closePane packagePane >> return ()) ideR
+            else do
+                md <- messageDialogNew (Just window) []
+                    MessageQuestion
+                    ButtonsYesNo
+                    "Unsaved changes. Close anyway?"
+                set md [ windowWindowPosition := WinPosCenterOnParent ]
+                resp <- dialogRun md
+                widgetDestroy md
+                case resp of
+                    ResponseYes ->   do
+                        reflectIDE (closePane packagePane >> return ()) ideR
+                    _  ->   return ())
+    save `onClicked` (do
+        mbNewPackage' <- extract packageD [getExt]
+        case mbNewPackage' of
+            Nothing -> return ()
+            Just newPackage' -> let newPackage = fromEditor newPackage' in do
+                let packagePath = packageDir </> (display . pkgName . package . pd) newPackage'
+                                                ++ ".cabal"
+                writePackageDescription packagePath newPackage
+                reflectIDE (do
+                    afterSaveAction packagePath
                     closePane packagePane
-                    initPackage packageDir pde{bis = take (length (bis pde) - 1) (bis pde)}
-                        (packageDD
-                            packageInfos
-                            packageDir
-                            modules
-                            (length (bis pde) - 1)
-                            (concatMap (buildInfoD (Just packageDir) modules)
-                                [0..length (bis pde) - 2]))
-                        panePath nb modules afterSaveAction) ideR)
-    boxPackStart vb widget PackGrow 7
-    boxPackEnd vb bb PackNatural 7
+                    return ()) ideR)
+    registerEvent notifier MayHaveChanged (\ e -> do
+        mbP <- extract packageD [getExt]
+        let hasChanged = case mbP of
+                                Nothing -> False
+                                Just p -> p /= origPackageD
+        when (isJust mbP) $ labelSetMarkup label ""
+        when (isJust mbP) $ comparePDE (fromJust mbP) packageD
+        markLabel nb (getTopWidget packagePane) hasChanged
+        widgetSetSensitive save hasChanged
+        return (e{gtkReturn=False}))
+    registerEvent notifier ValidationError (\e -> do
+        labelSetMarkup label $ "<span foreground=\"red\" size=\"x-large\">The following fields have invalid values: "
+            ++ eventText e ++ "</span>"
+        return e)
     return (Just packagePane,[])
 
 -- ---------------------------------------------------------------------
@@ -426,7 +436,7 @@ packageDD packages fp modules numBuildInfos extras = NFD ([
            $ emptyParams)
             (synopsis . pd)
             (\ a b -> b{pd = (pd b){synopsis = a}})
-            (stringEditor (const True))
+            (stringEditor (const True) True)
     ,   mkField
             (paraName <<<- ParaName "Package Identifier" $ emptyParams)
             (package . pd)
@@ -445,24 +455,24 @@ packageDD packages fp modules numBuildInfos extras = NFD ([
             (paraName <<<- ParaName "Homepage" $ emptyParams)
             (homepage . pd)
             (\ a b -> b{pd = (pd b){homepage = a}})
-            (stringEditor (const True))
+            (stringEditor (const True) True)
     ,   mkField
             (paraName <<<- ParaName "Package URL" $ emptyParams)
             (pkgUrl . pd)
             (\ a b -> b{pd = (pd b){pkgUrl = a}})
-            (stringEditor (const True))
+            (stringEditor (const True) True)
     ,   mkField
             (paraName <<<- ParaName "Category" $ emptyParams)
             (category . pd)
             (\ a b -> b{pd = (pd b){category = a}})
-            (stringEditor (const True))
+            (stringEditor (const True) True)
     ]),
     ("Description", VFD emptyParams [
         mkField
             (paraName <<<- ParaName "Stability" $ emptyParams)
             (stability . pd)
             (\ a b -> b{pd = (pd b){stability = a}})
-            (stringEditor (const True))
+            (stringEditor (const True) True)
 #if MIN_VERSION_Cabal(1,8,0)
             -- TODO
 #else
@@ -481,22 +491,22 @@ packageDD packages fp modules numBuildInfos extras = NFD ([
             (paraName <<<- ParaName "Copyright" $ emptyParams)
             (copyright . pd)
             (\ a b -> b{pd = (pd b){copyright = a}})
-            (stringEditor (const True))
+            (stringEditor (const True) True)
     ,   mkField
             (paraName <<<- ParaName "Author" $ emptyParams)
             (author . pd)
             (\ a b -> b{pd = (pd b){author = a}})
-            (stringEditor (const True))
+            (stringEditor (const True) True)
     ,   mkField
             (paraName <<<- ParaName "Maintainer" $ emptyParams)
             (maintainer . pd)
             (\ a b -> b{pd = (pd b){maintainer = a}})
-            (stringEditor (const True))
+            (stringEditor (const True) True)
     ,   mkField
             (paraName <<<- ParaName "Bug Reports" $ emptyParams)
             (bugReports . pd)
             (\ a b -> b{pd = (pd b){bugReports = a}})
-            (stringEditor (const True))
+            (stringEditor (const True) True)
     ]),
 --    ("Repositories", VFD emptyParams [
 --        mkField
@@ -511,7 +521,7 @@ packageDD packages fp modules numBuildInfos extras = NFD ([
                     $ paraDirection <<<- ParaDirection Vertical
                         $ paraMinSize <<<- ParaMinSize (-1,250)
                             $ emptyParams)
-            (reverse . buildDepends . pd)
+            (buildDepends . pd)
             (\ a b -> b{pd = (pd b){buildDepends = a}})
             (dependenciesEditor packages)
     ]),
@@ -522,7 +532,11 @@ packageDD packages fp modules numBuildInfos extras = NFD ([
                     "Does this package depends on a specific version of Cabal?"
                     $ paraShadow <<<- ParaShadow ShadowIn $ emptyParams)
             (descCabalVersion . pd)
+#if MIN_VERSION_Cabal(1,10,0)
+            (\ a b -> b{pd = (pd b){specVersionRaw = Right a}})
+#else
             (\ a b -> b{pd = (pd b){descCabalVersion = a}})
+#endif
             versionRangeEditor
     ,   mkField
             (paraName <<<- ParaName "Tested with compiler"
@@ -596,7 +610,7 @@ packageDD packages fp modules numBuildInfos extras = NFD ([
                                    ,("Value",\(_,v) -> [cellText := v])])
                 ((pairEditor
                     (stringxEditor (const True),emptyParams)
-                    (stringEditor (const True),emptyParams)),emptyParams)
+                    (stringEditor (const True) True,emptyParams)),emptyParams)
             Nothing
             Nothing)
     ]),
@@ -706,8 +720,13 @@ buildInfoD fp modules i = [
                          $ paraMinSize <<<- ParaMinSize (-1,400)
                             $ paraPack <<<- ParaPack PackGrow
                                 $ emptyParams)
+#if MIN_VERSION_Cabal(1,10,0)
+            (oldExtensions . (\a -> a !! i) . bis)
+            (\ a b -> b{bis = update (bis b) i (\bi -> bi{oldExtensions = a})})
+#else
             (extensions . (\a -> a !! i) . bis)
             (\ a b -> b{bis = update (bis b) i (\bi -> bi{extensions = a})})
+#endif
             extensionsEditor
     ]),
     (show (i + 1) ++ " Build Tools ", VFD emptyParams [
@@ -750,7 +769,7 @@ buildInfoD fp modules i = [
                 $ paraDirection <<<- ParaDirection Vertical $ emptyParams)
             (includes . (\a -> a !! i) . bis)
             (\ a b -> b{bis = update (bis b) i (\bi -> bi{includes = a})})
-            (stringsEditor (const True))
+            (stringsEditor (const True) True)
      ,   mkField
             (paraName <<<- ParaName "A list of header files to install"
                 $ paraDirection <<<- ParaDirection Vertical $ emptyParams)
@@ -779,7 +798,7 @@ buildInfoD fp modules i = [
                 $ paraDirection <<<- ParaDirection Vertical $ emptyParams)
             (extraLibs . (\a -> a !! i) . bis)
             (\ a b -> b{bis = update (bis b) i (\bi -> bi{extraLibs = a})})
-            (stringsEditor (const True))
+            (stringsEditor (const True) True)
      ,   mkField
             (paraName <<<- ParaName "A list of directories to search for libraries."
                 $ paraDirection <<<- ParaDirection Vertical $ emptyParams)
@@ -793,7 +812,7 @@ buildInfoD fp modules i = [
                 $ paraDirection <<<- ParaDirection Vertical $ emptyParams)
             (frameworks . (\a -> a !! i) . bis)
             (\ a b -> b{bis = update (bis b) i (\bi -> bi{frameworks = a})})
-            (stringsEditor (const True))
+            (stringsEditor (const True) True)
     ,   mkField
             (paraName <<<- ParaName "Custom fields build info"
                 $ paraShadow <<<- ParaShadow ShadowIn
@@ -805,14 +824,14 @@ buildInfoD fp modules i = [
                                    ,("Value",\(_,v) -> [cellText := v])])
                 ((pairEditor
                     (stringxEditor (const True),emptyParams)
-                    (stringEditor (const True),emptyParams)),emptyParams)
+                    (stringEditor (const True) True,emptyParams)),emptyParams)
             Nothing
             Nothing)
             ])]
 
 stringxEditor :: (String -> Bool) -> Editor String
 stringxEditor val para noti = do
-    (wid,inj,ext) <- stringEditor val para noti
+    (wid,inj,ext) <- stringEditor val True para noti
     let
         xinj ("") = inj ""
         xinj ('x':'-':rest) = inj rest
@@ -826,7 +845,7 @@ stringxEditor val para noti = do
 
 optsEditor :: Editor [String]
 optsEditor para noti = do
-    (wid,inj,ext) <- stringEditor (const True) para noti
+    (wid,inj,ext) <- stringEditor (const True) True para noti
     let
         oinj = inj . unwords
         oext = do
@@ -839,7 +858,7 @@ optsEditor para noti = do
 packageEditor :: Editor PackageIdentifier
 packageEditor para noti = do
     (wid,inj,ext) <- pairEditor
-        (stringEditor (\s -> not (null s)), paraName <<<- ParaName "Name" $ emptyParams)
+        (stringEditor (\s -> not (null s)) True, paraName <<<- ParaName "Name" $ emptyParams)
         (versionEditor, paraName <<<- ParaName "Version" $ emptyParams)
         (paraDirection <<<- ParaDirection Horizontal
             $ paraShadow <<<- ParaShadow ShadowIn
@@ -872,7 +891,7 @@ compilerFlavorEditor :: Editor CompilerFlavor
 compilerFlavorEditor para noti = do
     (wid,inj,ext) <- eitherOrEditor
         (comboSelectionEditor flavors show, paraName <<<- (ParaName "Select compiler") $ emptyParams)
-        (stringEditor (\s -> not (null s)), paraName <<<- (ParaName "Specify compiler") $ emptyParams)
+        (stringEditor (\s -> not (null s)) True, paraName <<<- (ParaName "Specify compiler") $ emptyParams)
         "Other"
         (paraName <<<- ParaName "Select" $ para)
         noti
@@ -893,7 +912,7 @@ buildTypeEditor :: Editor BuildType
 buildTypeEditor para noti = do
     (wid,inj,ext) <- eitherOrEditor
         (comboSelectionEditor flavors show, paraName <<<- (ParaName "Select") $ emptyParams)
-        (stringEditor (const True), paraName <<<- (ParaName "Unknown") $ emptyParams)
+        (stringEditor (const True) True, paraName <<<- (ParaName "Unknown") $ emptyParams)
         "Unknown"
         (paraName <<<- ParaName "Select" $ para)
         noti
@@ -951,11 +970,11 @@ repoEditor paras noti = do
     (widg,inj,ext) <- tupel7Editor
                             (repoKindEditor,noBorder)
                             (maybeEditor (repoTypeEditor,noBorder) True "Specify a type", emptyParams)
-                            (maybeEditor (stringEditor (const True),noBorder) True "Specify a location", emptyParams)
-                            (maybeEditor (stringEditor (const True),noBorder) True "Specify a module", emptyParams)
-                            (maybeEditor (stringEditor (const True),noBorder) True "Specify a branch", emptyParams)
-                            (maybeEditor (stringEditor (const True),noBorder) True "Specify a tag", emptyParams)
-                            (maybeEditor (stringEditor (const True),noBorder) True "Specify a subdir", emptyParams)
+                            (maybeEditor (stringEditor (const True) True,noBorder) True "Specify a location", emptyParams)
+                            (maybeEditor (stringEditor (const True) True,noBorder) True "Specify a module", emptyParams)
+                            (maybeEditor (stringEditor (const True) True,noBorder) True "Specify a branch", emptyParams)
+                            (maybeEditor (stringEditor (const True) True,noBorder) True "Specify a tag", emptyParams)
+                            (maybeEditor (stringEditor (const True) True,noBorder) True "Specify a subdir", emptyParams)
                             (paraDirection  <<<- ParaDirection Vertical $ noBorder)
                             noti
     return (widg,
@@ -975,7 +994,7 @@ repoKindEditor :: Editor RepoKind
 repoKindEditor paras noti = do
     (widg,inj,ext) <- pairEditor
                         (comboSelectionEditor selectionList show, emptyParams)
-                        (stringEditor (const True),emptyParams)
+                        (stringEditor (const True) True,emptyParams)
                         paras
                         noti
     return (widg,
@@ -994,7 +1013,7 @@ repoTypeEditor :: Editor RepoType
 repoTypeEditor paras noti = do
     (widg,inj,ext) <- pairEditor
                         (comboSelectionEditor selectionList show, emptyParams)
-                        (stringEditor (const True),emptyParams)
+                        (stringEditor (const True) True,emptyParams)
                         paras
                         noti
     return (widg,
@@ -1041,7 +1060,7 @@ libraryEditor fp modules numBuildInfos para noti = do
             paraName <<<- ParaName "Exposed"
             $ paraSynopsis <<<- ParaSynopsis "Is the lib to be exposed by default?"
             $ emptyParams)
-            (modulesEditor modules,
+            (modulesEditor (sort modules),
             paraName <<<- ParaName "Exposed Modules"
             $ paraMinSize <<<- ParaMinSize (-1,300)
             $ para)
@@ -1081,10 +1100,10 @@ executablesEditor fp modules countBuildInfo p =
 executableEditor :: Maybe FilePath -> [ModuleName] -> Int -> Editor Executable'
 executableEditor fp modules countBuildInfo para noti = do
     (wid,inj,ext) <- tupel3Editor
-        (stringEditor (\s -> not (null s)),
+        (stringEditor (\s -> not (null s)) True,
             paraName <<<- ParaName "Executable Name"
             $ emptyParams)
-        (stringEditor (\s -> not (null s)),
+        (stringEditor (\s -> not (null s)) True,
             paraDirection <<<- ParaDirection Vertical
             $ paraName <<<- ParaName "File with main function"
             $ emptyParams)
