@@ -28,10 +28,12 @@ module IDE.Pane.Preferences (
 ) where
 
 import Graphics.UI.Gtk
-       (cellText, widgetModifyFont, onClicked, boxPackEnd, boxPackStart,
+       (widgetDestroy, dialogRun, windowWindowPosition, dialogAddButton,
+        messageDialogNew, labelSetMarkup, labelNew, widgetSetSensitive,
+        cellText, widgetModifyFont, onClicked, boxPackEnd, boxPackStart,
         buttonNewFromStock, hButtonBoxNew, vBoxNew, castToWidget, VBox,
         ShadowType(..), Packing(..), fontDescriptionFromString, AttrOp(..),
-        FileChooserAction(..), Color(..))
+        FileChooserAction(..), Color(..), ResponseId(..))
 import Control.Monad.Reader
 import qualified Text.PrettyPrint.HughesPJ as PP
 import Distribution.Package
@@ -50,7 +52,6 @@ import Text.PrinterParser hiding (fieldParser,parameters)
 import IDE.TextEditor
 import IDE.Pane.SourceBuffer
 import IDE.Pane.Log
-import Default
 import IDE.Utils.FileUtils
 import IDE.Utils.GUIUtils
 import IDE.Debug
@@ -61,16 +62,21 @@ import IDE.Debug
 import Graphics.UI.Gtk.SourceView
     (sourceStyleSchemeManagerGetSchemeIds, sourceStyleSchemeManagerNew)
 import System.Time (getClockTime)
-import System.FilePath((</>))
 import qualified IDE.StrippedPrefs as SP
 import Control.Exception(SomeException,catch)
 import Prelude hiding(catch)
+import Data.List (sortBy)
+import Data.Maybe (isJust)
+import Graphics.UI.Gtk.Windows.MessageDialog
+       (ButtonsType(..), MessageType(..))
+import System.Glib.Attributes (set)
+import Graphics.UI.Gtk.General.Enums (WindowPosition(..))
 
 -- ---------------------------------------------------------------------
 -- This needs to be incremented, when the preferences format changes
 --
 prefsVersion :: Int
-prefsVersion = 1
+prefsVersion = 2
 
 --
 -- | The Preferences Pane
@@ -104,15 +110,18 @@ instance RecoverablePane IDEPrefs PrefsState IDEM where
             bb      <-  hButtonBoxNew
             apply   <-  buttonNewFromStock "gtk-apply"
             restore <-  buttonNewFromStock "Restore"
+            closeB  <-  buttonNewFromStock "gtk-cancel"
             save    <-  buttonNewFromStock "gtk-save"
-            closeB  <-  buttonNewFromStock "gtk-close"
+            widgetSetSensitive save False
             boxPackStart bb apply PackNatural 0
             boxPackStart bb restore PackNatural 0
-            boxPackStart bb save PackNatural 0
-            boxPackStart bb closeB PackNatural 0
+            boxPackEnd bb closeB PackNatural 0
+            boxPackEnd bb save PackNatural 0
             (widget,injb,ext,notifier) <-  buildEditor
                                 (extractFieldDescription $ prefsDescription configDir packageInfos) prefs
             boxPackStart vb widget PackGrow 7
+            label   <-  labelNew Nothing
+            boxPackStart vb label PackNatural 0
             boxPackEnd vb bb PackNatural 7
             let prefsPane = IDEPrefs vb
             apply `onClicked` (do
@@ -127,29 +136,66 @@ instance RecoverablePane IDEPrefs PrefsState IDEM where
                 lastAppliedPrefs <- readIORef lastAppliedPrefsRef
                 mapM_ (\ (FDPP _ _ _ _ applyF) -> reflectIDE (applyF prefs lastAppliedPrefs) ideR ) flatPrefsDesc
                 injb prefs
-                writeIORef lastAppliedPrefsRef prefs)
+                writeIORef lastAppliedPrefsRef prefs
+                markLabel nb (getTopWidget prefsPane) False
+                widgetSetSensitive save False
+                )
             save `onClicked` (do
                 lastAppliedPrefs <- readIORef lastAppliedPrefsRef
                 mbNewPrefs <- extract prefs [ext]
                 case mbNewPrefs of
                     Nothing -> return ()
                     Just newPrefs -> do
-                    mapM_ (\ (FDPP _ _ _ _ applyF) -> reflectIDE (applyF newPrefs lastAppliedPrefs) ideR ) flatPrefsDesc
-                    fp   <- getConfigFilePathForSave standardPreferencesFilename
-                    writePrefs fp newPrefs
-                    fp2  <-  getConfigFilePathForSave strippedPreferencesFilename
-                    SP.writeStrippedPrefs fp2
-                        (SP.Prefs {SP.sourceDirectories = sourceDirectories newPrefs,
-                                   SP.unpackDirectory   = unpackDirectory newPrefs,
-                                   SP.retrieveURL       = retrieveURL newPrefs,
-                                   SP.retrieveStrategy  = retrieveStrategy newPrefs,
-                                   SP.serverPort        = serverPort newPrefs,
-                                   SP.endWithLastConn   = endWithLastConn newPrefs})
-                    reflectIDE (modifyIDE_ (\ide -> ide{prefs = newPrefs})) ideR )
-            closeB `onClicked` (reflectIDE (closePane prefsPane >> return ()) ideR )
-            registerEvent notifier FocusIn (Left (\e -> do
+                        mapM_ (\ (FDPP _ _ _ _ applyF) -> reflectIDE (applyF newPrefs lastAppliedPrefs) ideR ) flatPrefsDesc
+                        fp   <- getConfigFilePathForSave standardPreferencesFilename
+                        writePrefs fp newPrefs
+                        fp2  <-  getConfigFilePathForSave strippedPreferencesFilename
+                        SP.writeStrippedPrefs fp2
+                            (SP.Prefs {SP.sourceDirectories = sourceDirectories newPrefs,
+                                       SP.unpackDirectory   = unpackDirectory newPrefs,
+                                       SP.retrieveURL       = retrieveURL newPrefs,
+                                       SP.retrieveStrategy  = retrieveStrategy newPrefs,
+                                       SP.serverPort        = serverPort newPrefs,
+                                       SP.endWithLastConn   = endWithLastConn newPrefs})
+                        reflectIDE (modifyIDE_ (\ide -> ide{prefs = newPrefs})) ideR
+                        reflectIDE (closePane prefsPane >> return ()) ideR)
+            closeB `onClicked` do
+                mbP <- extract prefs [ext]
+                let hasChanged = case mbP of
+                                        Nothing -> False
+                                        Just p -> p{prefsFormat = 0, prefsSaveTime = ""} /=
+                                                  prefs{prefsFormat = 0, prefsSaveTime = ""}
+                if not hasChanged
+                    then reflectIDE (closePane prefsPane >> return ()) ideR
+                    else do
+                        md <- messageDialogNew (Just windows) []
+                            MessageQuestion
+                            ButtonsYesNo
+                            "Unsaved changes. Close anyway?"
+                        set md [ windowWindowPosition := WinPosCenterOnParent ]
+                        resp <- dialogRun md
+                        widgetDestroy md
+                        case resp of
+                            ResponseYes ->   do
+                                reflectIDE (closePane prefsPane >> return ()) ideR
+                            _  ->   return ()
+            registerEvent notifier FocusIn (\e -> do
                 reflectIDE (makeActive prefsPane) ideR
-                return (e{gtkReturn=False})))
+                return (e{gtkReturn=False}))
+            registerEvent notifier MayHaveChanged (\ e -> do
+                mbP <- extract prefs [ext]
+                let hasChanged = case mbP of
+                                        Nothing -> False
+                                        Just p -> p{prefsFormat = 0, prefsSaveTime = ""} /=
+                                                  prefs{prefsFormat = 0, prefsSaveTime = ""}
+                when (isJust mbP) $ labelSetMarkup label ""
+                markLabel nb (getTopWidget prefsPane) hasChanged
+                widgetSetSensitive save hasChanged
+                return (e{gtkReturn=False}))
+            registerEvent notifier ValidationError (\e -> do
+                labelSetMarkup label $ "<span foreground=\"red\" size=\"x-large\">The following fields have invalid values: "
+                    ++ eventText e ++ "</span>"
+                return e)
             return (Just prefsPane,[])
 
 getPrefs :: Maybe PanePath -> IDEM IDEPrefs
@@ -209,17 +255,19 @@ prefsDescription configDir packages = NFDPP [
             (paraName <<<- ParaName "Right margin"
                 $ paraSynopsis <<<- ParaSynopsis "Size or 0 for no right margin"
                     $ paraShadow <<<- ParaShadow ShadowIn $ emptyParams)
-            (\a -> (PP.text . show) (case a of Nothing -> 0; Just i -> i))
-            (do i <- intParser
-                return (if i == 0 then Nothing else Just i))
+            (PP.text . show)
+            readParser
             rightMargin
             (\b a -> a{rightMargin = b})
-            (maybeEditor (intEditor (1.0, 200.0, 5.0), paraName <<<- ParaName "Position"
+            (disableEditor (intEditor (1.0, 200.0, 5.0), paraName <<<- ParaName "Position"
                     $ emptyParams)
                     True "Show it ?")
             (\b -> do
                 buffers <- allBuffers
-                mapM_ (\buf -> setRightMargin (sourceView buf) b) buffers)
+                mapM_ (\buf -> setRightMargin (sourceView buf)
+                                (case b of
+                                    (True,v) -> Just v
+                                    (False,_) -> Nothing)) buffers)
     ,   mkFieldPP
             (paraName <<<- ParaName "Tab width" $ emptyParams)
             (PP.text . show)
@@ -230,6 +278,16 @@ prefsDescription configDir packages = NFDPP [
             (\i -> do
                 buffers <- allBuffers
                 mapM_ (\buf -> setIndentWidth (sourceView buf) i) buffers)
+    ,   mkFieldPP
+            (paraName <<<- ParaName "Wrap lines" $ emptyParams)
+            (PP.text . show)
+            boolParser
+            wrapLines
+            (\b a -> a{wrapLines = b})
+            boolEditor
+            (\b -> do
+                buffers <- allBuffers
+                mapM_ (\buf -> setWrapMode (sourceView buf) b) buffers)
     ,   mkFieldPP
             (paraName <<<- ParaName "Use standard line ends even on windows" $ emptyParams)
             (PP.text . show)
@@ -251,25 +309,25 @@ prefsDescription configDir packages = NFDPP [
                 $ paraSynopsis <<<- ParaSynopsis
                     "Empty for do not use or the name of a candy file in a config dir"
                     $ paraShadow <<<- ParaShadow ShadowIn $ emptyParams)
-            (\a -> PP.text (case a of Nothing -> show ""; Just s -> show s))
-            (do str <- stringParser
-                return (if null str then Nothing else Just (str)))
+            (PP.text . show)
+            readParser
             sourceCandy (\b a -> a{sourceCandy = b})
-            (maybeEditor ((stringEditor (\s -> not (null s))), paraName <<<- ParaName "Candy specification"
+            (disableEditor (stringEditor (\s -> not (null s)) True,
+                paraName <<<- ParaName "Candy specification"
                                     $ emptyParams)
                     True "Use it ?")
             (\cs -> case cs of
-                        Nothing -> do
+                        (False,_) -> do
                             setCandyState False
                             editCandy
-                        Just name -> do
+                        (True,name) -> do
                             setCandyState True
                             editCandy)
     ,   mkFieldPP
             (paraName <<<- ParaName "Editor Style" $ emptyParams)
-            (\a -> PP.text (case a of Nothing -> show ""; Just s -> show s))
+            (\a -> PP.text (case a of (False,_) -> show ""; (True, s) -> show s))
             (do str <- stringParser
-                return (if null str then Nothing else Just (str)))
+                return (if null str then (False,"classic") else (True,str)))
             sourceStyle
             (\b a -> a{sourceStyle = b})
             styleEditor
@@ -277,7 +335,9 @@ prefsDescription configDir packages = NFDPP [
                 buffers <- allBuffers
                 mapM_ (\buf -> do
                     ebuf <- getBuffer (sourceView buf)
-                    setStyle ebuf mbs) buffers)
+                    setStyle ebuf (case mbs of
+                                    (False,_) -> Nothing
+                                    (True,s) -> Just s)) buffers)
     ,   mkFieldPP
             (paraName <<<- ParaName "Found Text Background" $ emptyParams)
             (PP.text . show)
@@ -322,6 +382,14 @@ prefsDescription configDir packages = NFDPP [
                     tagTable <- getTagTable ebuf
                     --  TODO find and set the tag background
                     return ())
+    ,   mkFieldPP
+            (paraName <<<- ParaName "Automatically load modified files modified outside of Leksah" $ emptyParams)
+            (PP.text . show)
+            boolParser
+            autoLoad
+            (\b a -> a{autoLoad = b})
+            boolEditor
+            (\i -> return ())
     ,   mkFieldPP
             (paraName <<<- ParaName "Use Yi - Experimental feature (could wipe your files)" $ emptyParams)
             (PP.text . show)
@@ -382,7 +450,7 @@ prefsDescription configDir packages = NFDPP [
             identifier
             keymapName
             (\b a -> a{keymapName = b})
-            (stringEditor (\s -> not (null s)))
+            (stringEditor (\s -> not (null s)) True)
             (\ a -> return ())
     ]),
     ("Initial Pane positions", VFDPP emptyParams [
@@ -390,7 +458,9 @@ prefsDescription configDir packages = NFDPP [
             (paraName <<<- ParaName
                 "Categories for panes"
                 $ paraShadow <<<- ParaShadow ShadowIn
-                     $ paraDirection <<<- ParaDirection Vertical $ emptyParams)
+                     $ paraDirection <<<- ParaDirection Vertical
+                        $ paraMinSize <<<- ParaMinSize (-1,130)
+                            $ emptyParams)
             (PP.text . show)
             readParser
             categoryForPane
@@ -399,16 +469,18 @@ prefsDescription configDir packages = NFDPP [
                 (ColumnDescr True [("Pane Id",\(n,_) -> [cellText := n])
                                    ,("Pane Category",\(_,v) -> [cellText := v])])
                 ((pairEditor
-                    (stringEditor (\s -> not (null s)),emptyParams)
-                    (stringEditor (\s -> not (null s)),emptyParams)),emptyParams)
-            Nothing
-            Nothing)
+                    (stringEditor (\s -> not (null s)) True,emptyParams)
+                    (stringEditor (\s -> not (null s)) True,emptyParams)),emptyParams)
+                (Just (sortBy (\(a,_) (a2,_) -> compare a a2)))
+                (Just (\(a,_) (a2,_) -> a == a2)))
             (\i -> return ())
     ,   mkFieldPP
             (paraName <<<- ParaName
                 "Pane path for category"
                 $ paraShadow <<<- ParaShadow ShadowIn
-                     $ paraDirection <<<- ParaDirection Vertical $ emptyParams)
+                     $ paraDirection <<<- ParaDirection Vertical
+                        $ paraMinSize <<<- ParaMinSize (-1,130)
+                            $ emptyParams)
             (PP.text . show)
             readParser
             pathForCategory
@@ -417,10 +489,10 @@ prefsDescription configDir packages = NFDPP [
                 (ColumnDescr True [("Pane category",\(n,_) -> [cellText := n])
                                    ,("Pane path",\(_,v) -> [cellText := show v])])
                 ((pairEditor
-                    (stringEditor (\s -> not (null s)),emptyParams)
+                    (stringEditor (\s -> not (null s)) True,emptyParams)
                     (genericEditor,emptyParams)),emptyParams)
-            Nothing
-            Nothing)
+                (Just (sortBy (\(a,_) (a2,_) -> compare a a2)))
+                (Just (\(a,_) (a2,_) -> a == a2)))
             (\i -> return ())
     ,   mkFieldPP
             (paraName <<<- ParaName "Default pane path" $ emptyParams)
@@ -434,7 +506,9 @@ prefsDescription configDir packages = NFDPP [
     ("Metadata", VFDPP emptyParams [
         mkFieldPP
             (paraName <<<- ParaName
-                "Paths under which haskell sources for packages may be found" $ emptyParams)
+                "Paths under which haskell sources for packages may be found"
+                        $ paraMinSize <<<- ParaMinSize (-1,100)
+                            $ emptyParams)
             (PP.text . show)
             readParser
             sourceDirectories
@@ -442,28 +516,28 @@ prefsDescription configDir packages = NFDPP [
             (filesEditor Nothing FileChooserActionSelectFolder "Select folder")
             (\i -> return ())
     ,   mkFieldPP
-            (paraName <<<- ParaName "Maybe a directory for unpacking cabal packages" $ emptyParams)
+            (paraName <<<- ParaName "Unpack source for cabal packages to" $ emptyParams)
             (PP.text . show)
             readParser
             unpackDirectory
             (\b a -> a{unpackDirectory = b})
-            (maybeEditor (stringEditor (\ _ -> True),emptyParams) True "")
+            (maybeEditor (stringEditor (\ _ -> True) True,emptyParams) True "")
             (\i -> return ())
     ,   mkFieldPP
-            (paraName <<<- ParaName "An URL to load prebuild metadata" $ emptyParams)
+            (paraName <<<- ParaName "URL from which to download prebuilt metadata" $ emptyParams)
             (PP.text . show)
             stringParser
             retrieveURL
             (\b a -> a{retrieveURL = b})
-            (stringEditor (\ _ -> True))
+            (stringEditor (const True) True)
             (\i -> return ())
     ,   mkFieldPP
-            (paraName <<<- ParaName "A strategy for downloading prebuild metadata" $ emptyParams)
+            (paraName <<<- ParaName "Strategy for downloading prebuilt metadata" $ emptyParams)
             (PP.text . show)
             readParser
             retrieveStrategy
             (\b a -> a{retrieveStrategy = b})
-            (enumEditor ["Retrieve then build","Build then retrieve","Never retrieve"])
+            (enumEditor ["Try to download and then build locally if that fails","Try to build locally and then download if that fails","Never download (just try to build locally)"])
             (\i -> return ())
     ,   mkFieldPP
             (paraName <<<- ParaName "Update metadata at startup" $ emptyParams)
@@ -474,7 +548,7 @@ prefsDescription configDir packages = NFDPP [
             boolEditor
             (\i -> return ())
     ,   mkFieldPP
-            (paraName <<<- ParaName "Port number for server connection" $ emptyParams)
+            (paraName <<<- ParaName "Port number for leksah to comunicate with leksah-server" $ emptyParams)
             (PP.text . show)
             intParser
             serverPort
@@ -482,15 +556,15 @@ prefsDescription configDir packages = NFDPP [
             (intEditor (1.0, 65535.0, 1.0))
             (\i -> return ())
     ,   mkFieldPP
-            (paraName <<<- ParaName "Server IP address " $ emptyParams)
+            (paraName <<<- ParaName "IP address for leksah to comunicate with leksah-server" $ emptyParams)
             (PP.text . show)
             stringParser
             serverIP
             (\b a -> a{serverIP = b})
-            (stringEditor (\ s -> not $ null s))
+            (stringEditor (\ s -> not $ null s) True)
             (\i -> return ())
     ,   mkFieldPP
-            (paraName <<<- ParaName "End the server with last connection" $ emptyParams)
+            (paraName <<<- ParaName "Stop the leksah-server process when leksah disconnects" $ emptyParams)
             (PP.text . show)
             boolParser
             endWithLastConn
@@ -501,7 +575,9 @@ prefsDescription configDir packages = NFDPP [
     ("Blacklist", VFDPP emptyParams [
         mkFieldPP
             (paraName <<<- ParaName
-                "Packages which are excluded from the modules pane" $ emptyParams)
+                "Packages which are excluded from the modules pane"
+                        $ paraMinSize <<<- ParaMinSize (-1,200)
+                            $ emptyParams)
             (PP.text . show)
             readParser
             packageBlacklist
@@ -527,24 +603,31 @@ prefsDescription configDir packages = NFDPP [
             boolEditor
             (\i -> return ())
          , mkFieldPP
-            (paraName <<<- ParaName "Include linking in background builds" $ emptyParams)
+            (paraName <<<- ParaName "Make mode" $ emptyParams)
             (PP.text . show)
             boolParser
-            backgroundLink
-            (\b a -> a{backgroundLink = b})
+            makeMode
+            (\b a -> a{makeMode = b})
+            (boolEditor2 "Single mode")
+            (\i -> return ())
+
+         , mkFieldPP
+            (paraName <<<- ParaName "Single build without linking" $ emptyParams)
+            (PP.text . show)
+            boolParser
+            singleBuildWithoutLinking
+            (\b a -> a{singleBuildWithoutLinking = b})
             boolEditor
             (\i -> return ())
          , mkFieldPP
-            (paraName <<<- ParaName "Auto install packages" $
-                paraShadow <<<- ParaShadow ShadowIn $emptyParams)
+            (paraName <<<- ParaName "Don't install last package" $ emptyParams)
             (PP.text . show)
-            readParser
-            autoInstall
-            (\b a -> a{autoInstall = b})
-            (enumEditor ["Install always after a succesful build",
-                "Install if it's a library with dependend packages in the workspace",
-                "Never install"])
+            boolParser
+            dontInstallLast
+            (\b a -> a{dontInstallLast = b})
+            boolEditor
             (\i -> return ())
+
     ]),
     ("Debug", VFDPP emptyParams [
            mkFieldPP
@@ -587,7 +670,7 @@ prefsDescription configDir packages = NFDPP [
             stringParser
             browser
             (\b a -> a{browser = b})
-            (stringEditor (\s -> not (null s)))
+            (stringEditor (\s -> not (null s)) True)
             (\i -> return ())
     ,   mkFieldPP
             (paraName <<<- ParaName "URL for searching documentation" $
@@ -599,51 +682,54 @@ prefsDescription configDir packages = NFDPP [
             stringParser
             docuSearchURL
             (\b a -> a{docuSearchURL = b})
-            (stringEditor (\s -> not (null s)))
+            (stringEditor (\s -> not (null s)) True)
             (\i -> return ())
     ])]
 
 
-styleEditor :: Editor (Maybe String)
+styleEditor :: Editor (Bool, String)
 styleEditor p n = do
     styleManager <- sourceStyleSchemeManagerNew
     ids          <- sourceStyleSchemeManagerGetSchemeIds styleManager
-    maybeEditor (comboSelectionEditor ids id, p) True "Select a special style?" p n
+    disableEditor (comboSelectionEditor ids id, p) True "Select a special style?" p n
 
 
 defaultPrefs = Prefs {
         prefsFormat         =   prefsVersion
     ,   prefsSaveTime       =   ""
     ,   showLineNumbers     =   True
-    ,   rightMargin         =   Just 100
+    ,   rightMargin         =   (True,100)
     ,   tabWidth            =   4
-    ,   sourceCandy         =   Just("candy")
+    ,   wrapLines           =   False
+    ,   sourceCandy         =   (False,"candy")
     ,   keymapName          =   "keymap"
     ,   forceLineEnds       =   True
     ,   removeTBlanks       =   True
     ,   textviewFont        =   Nothing
-    ,   sourceStyle         =   Nothing
+    ,   sourceStyle         =   (False,"classic")
     ,   foundBackground     =   Color 65535 65535 32768
     ,   contextBackground   =   Color 65535 49152 49152
     ,   breakpointBackground =  Color 65535 49152 32768
     ,   useYi               =   False
+    ,   autoLoad            =   False
     ,   logviewFont         =   Nothing
     ,   defaultSize         =   (1024,800)
     ,   browser             =   "firefox"
     ,   sourceDirectories   =   []
     ,   packageBlacklist    =   []
     ,   pathForCategory     =   [   ("EditorCategory",[SplitP (LeftP)])
+                                ,   ("LogCategory",[SplitP (RightP), SplitP (BottomP)])
                                 ,   ("ToolCategory",[SplitP (RightP),SplitP (TopP)])
-                                ,   ("LogCategory",[SplitP (RightP), SplitP (BottomP)])]
+                                ]
     ,   defaultPath         =   [SplitP (LeftP)]
     ,   categoryForPane     =   [   ("*ClassHierarchy","ToolCategory")
                                 ,   ("*Debug","ToolCategory")
+                                ,   ("*Flags","ToolCategory")
                                 ,   ("*Grep","ToolCategory")
                                 ,   ("*Info","ToolCategory")
                                 ,   ("*Log","LogCategory")
                                 ,   ("*Modules","ToolCategory")
                                 ,   ("*Package","ToolCategory")
-                                ,   ("*Flags","ToolCategory")
                                 ,   ("*Prefs","ToolCategory")
                                 ,   ("*References","ToolCategory")
                                 ,   ("*Search","ToolCategory")]
@@ -652,17 +738,13 @@ defaultPrefs = Prefs {
     ,   retrieveURL         =   "http://www.leksah.org"
     ,   retrieveStrategy    =   SP.RetrieveThenBuild
     ,   useCtrlTabFlipping  =   True
-    ,   docuSearchURL       =   "http://holumbus.fh-wedel.de/hayoo/hayoo.html?query="
+    ,   docuSearchURL       =   "http://www.holumbus.org/hayoo/hayoo.html?query="
     ,   completeRestricted  =   False
     ,   saveAllBeforeBuild  =   True
     ,   backgroundBuild     =   True
-    ,   backgroundLink      =
-#if defined(mingw32_HOST_OS) || defined(__MINGW32__)
-                                False
-#else
-                                True
-#endif
-    ,   autoInstall         =   InstallLibs
+    ,   makeMode            =   True
+    ,   singleBuildWithoutLinking  = False
+    ,   dontInstallLast     =   False
     ,   printEvldWithShow   =   True
     ,   breakOnException    =   True
     ,   breakOnError        =   True
