@@ -1,5 +1,5 @@
 {-# OPTIONS_GHC  -XDeriveDataTypeable -XMultiParamTypeClasses -XTypeSynonymInstances
-    -XScopedTypeVariables #-}
+    -XScopedTypeVariables -XRankNTypes #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Pane.SourceBuffer
@@ -94,28 +94,28 @@ import IDE.Completion as Completion (complete,cancel)
 import IDE.TextEditor
 import qualified System.IO.UTF8 as UTF8
 import Data.IORef (writeIORef,readIORef,newIORef,IORef(..))
-import Data.Char (isAlphaNum)
+import Data.Char (isAlphaNum, isSymbol)
 import Control.Event (triggerEvent)
 import IDE.Metainfo.Provider (getSystemInfo, getWorkspaceInfo)
 import Distribution.Text (simpleParse)
 import Distribution.ModuleName (ModuleName)
 import qualified Data.Set as Set (member)
-import Graphics.UI.Gtk.Gdk.Events as Gtk
 import Graphics.UI.Gtk
-       (Notebook, clipboardGet, atomNew, dialogAddButton, widgetDestroy,
+       (Notebook, clipboardGet, selectionClipboard, dialogAddButton, widgetDestroy,
         fileChooserGetFilename, widgetShow, fileChooserDialogNew,
         notebookGetNthPage, notebookPageNum, widgetHide, dialogRun,
         messageDialogNew, postGUIAsync, scrolledWindowSetShadowType,
-        scrolledWindowSetPolicy, castToWidget, ScrolledWindow)
+        scrolledWindowSetPolicy, castToWidget, ScrolledWindow, dialogSetDefaultResponse,
+        fileChooserSetCurrentFolder, fileChooserSelectFilename)
 import System.Glib.MainLoop (priorityDefaultIdle, idleAdd)
 #if MIN_VERSION_gtk(0,10,5)
-import Graphics.UI.Gtk (Underline(..))
+import Graphics.UI.Gtk (windowWindowPosition, Underline(..))
 #else
 import Graphics.UI.Gtk.Pango.Types (Underline(..))
 #endif
 import qualified Graphics.UI.Gtk as Gtk (Window, Notebook)
 import Graphics.UI.Gtk.General.Enums
-       (ShadowType(..), PolicyType(..))
+       (WindowPosition(..), ShadowType(..), PolicyType(..))
 import Graphics.UI.Gtk.Windows.MessageDialog
        (ButtonsType(..), MessageType(..))
 #if MIN_VERSION_gtk(0,10,5)
@@ -125,30 +125,11 @@ import Graphics.UI.Gtk.General.Structs (ResponseId(..))
 #endif
 import Graphics.UI.Gtk.Selectors.FileChooser
        (FileChooserAction(..))
+import System.Glib.Attributes (AttrOp(..), set)
+import IDE.BufferMode
 
-
---
--- | A text editor pane description
---
-data IDEBuffer      =   IDEBuffer {
-    fileName        ::  Maybe FilePath
-,   bufferName      ::  String
-,   addedIndex      ::  Int
-,   sourceView      ::  EditorView
-,   scrolledWindow  ::  ScrolledWindow
-,   modTime         ::  IORef (Maybe (ClockTime))
-} deriving (Typeable)
-
-data BufferState            =   BufferState FilePath Int
-                            |   BufferStateTrans String String Int
-    deriving(Eq,Ord,Read,Show,Typeable)
-
-instance Pane IDEBuffer IDEM
-    where
-    primPaneName    =   bufferName
-    getAddedIndex   =   addedIndex
-    getTopWidget    =   castToWidget . scrolledWindow
-    paneId b        =   ""
+allBuffers :: IDEM [IDEBuffer]
+allBuffers = getPanes
 
 instance RecoverablePane IDEBuffer BufferState IDEM where
     saveState p     =   do  buf    <- getBuffer (sourceView p)
@@ -177,18 +158,17 @@ instance RecoverablePane IDEBuffer BufferState IDEM where
             Nothing -> return Nothing
     recoverState pp (BufferStateTrans bn text i) =   do
         mbbuf    <-  newTextBuffer pp bn Nothing
-        useCandy    <- getCandyState
         case mbbuf of
             Just buf -> do
-                candy'  <- readIDE candy
-                gtkBuf  <-  getBuffer (sourceView buf)
+                useCandy <- useCandyFor buf
+                candy'   <- readIDE candy
+                gtkBuf   <-  getBuffer (sourceView buf)
                 setText gtkBuf text
-
                 when useCandy $ transformToCandy candy' gtkBuf
-                iter    <-  getIterAtOffset gtkBuf i
+                iter     <-  getIterAtOffset gtkBuf i
                 placeCursor gtkBuf iter
-                mark    <-  getInsertMark gtkBuf
-                ideR    <- ask
+                mark     <-  getInsertMark gtkBuf
+                ideR     <- ask
                 liftIO $ idleAdd  (do
                     reflectIDE (scrollToMark (sourceView buf) mark 0.0 (Just (0.3,0.3))) ideR
                     return False) priorityDefaultIdle
@@ -213,8 +193,6 @@ instance RecoverablePane IDEBuffer BufferState IDEM where
     buildPane panePath notebook builder = return Nothing
     builder pp nb w =    return (Nothing,[])
 
-
-
 startComplete :: IDEAction
 startComplete = do
     mbBuf <- maybeActiveBuf
@@ -225,7 +203,7 @@ startComplete = do
 
 selectSourceBuf :: FilePath -> IDEM (Maybe IDEBuffer)
 selectSourceBuf fp = do
-    fpc <-  liftIO $ canonicalizePath fp
+    fpc <- liftIO $ myCanonicalizePath fp
     buffers <- allBuffers
     let buf = filter (\b -> case fileName b of
                                 Just fn -> equalFilePath fn fpc
@@ -242,20 +220,9 @@ selectSourceBuf fp = do
                     pp      <- getBestPathForId  "*Buffer"
                     nbuf <- newTextBuffer pp (takeFileName fpc) (Just fpc)
                     return nbuf
-                else return Nothing
-
-recentSourceBuffers :: IDEM [PaneName]
-recentSourceBuffers = do
-    recentPanes' <- readIDE recentPanes
-    mbBufs       <- mapM mbPaneFromName recentPanes'
-    return $ map paneName ((catMaybes $ map (\ (PaneC p) -> cast p) $ catMaybes mbBufs) :: [IDEBuffer])
-
-lastActiveBufferPane :: IDEM (Maybe PaneName)
-lastActiveBufferPane = do
-    rs <- recentSourceBuffers
-    case rs of
-        (hd : _) -> return (Just hd)
-        _        -> return Nothing
+                else do
+                    ideMessage Normal ("File path not found " ++ fpc)
+                    return Nothing
 
 goToDefinition :: Descr -> IDEAction
 goToDefinition idDescr  = do
@@ -287,6 +254,7 @@ goToDefinition idDescr  = do
 
 goToSourceDefinition :: FilePath -> Maybe Location -> IDEAction
 goToSourceDefinition fp dscMbLocation = do
+    liftIO $ putStrLn $ "goToSourceDefinition " ++ fp
     mbBuf     <- selectSourceBuf fp
     when (isJust mbBuf && isJust dscMbLocation) $
         inActiveBufContext () $ \_ ebuf buf _ -> do
@@ -328,7 +296,7 @@ insertInBuffer idDescr = do
 
 markRefInSourceBuf :: Int -> IDEBuffer -> LogRef -> Bool -> IDEAction
 markRefInSourceBuf index buf logRef scrollTo = do
-    useCandy    <- getCandyState
+    useCandy    <- useCandyFor buf
     candy'      <- readIDE candy
     contextRefs <- readIDE contextRefs
     prefs       <- readIDE prefs
@@ -396,20 +364,6 @@ markRefInSourceBuf index buf logRef scrollTo = do
                 return False) priorityDefaultIdle
             return ()
 
-allBuffers :: IDEM [IDEBuffer]
-allBuffers = getPanes
-
-maybeActiveBuf :: IDEM (Maybe IDEBuffer)
-maybeActiveBuf = do
-    mbActivePane <- getActivePane
-    mbPane       <- lastActiveBufferPane
-    case (mbPane,mbActivePane) of
-        (Just paneName1, Just (paneName2,_)) | paneName1 == paneName2 -> do
-            (PaneC pane) <- paneFromName paneName1
-            let mbActbuf = cast pane
-            return mbActbuf
-        _ -> return Nothing
-
 newTextBuffer :: PanePath -> String -> Maybe FilePath -> IDEM (Maybe IDEBuffer)
 newTextBuffer panePath bn mbfn = do
     cont <- case mbfn of
@@ -454,7 +408,8 @@ builder' bs mbfn ind bn rbn ct prefs pp nb windows = do
     background foundTag $ foundBackground prefs
 
     beginNotUndoableAction buffer
-    when bs $ transformToCandy ct buffer
+    let mod = modFromFileName mbfn
+    when (bs && isHaskellMode mod) $ transformToCandy ct buffer
     endNotUndoableAction buffer
     setModified buffer False
     siter <- getStartIter buffer
@@ -464,17 +419,31 @@ builder' bs mbfn ind bn rbn ct prefs pp nb windows = do
     -- create a new SourceView Widget
     sv <- newView buffer (textviewFont prefs)
     setShowLineNumbers sv $ showLineNumbers prefs
-    setRightMargin sv $ rightMargin prefs
+    setRightMargin sv $ case rightMargin prefs of
+                            (False,_) -> Nothing
+                            (True,v) -> Just v
     setIndentWidth sv $ tabWidth prefs
-    setTabWidth sv $ tabWidth prefs
-    setStyle buffer $ sourceStyle prefs
+    setTabWidth sv $ 8 -- GHC treats tabs as 8 we should display them that way
+    drawTabs sv
+    setStyle buffer $ case sourceStyle prefs of
+                        (False,_) -> Nothing
+                        (True,v) -> Just v
 
     -- put it in a scrolled window
     sw <- getScrolledWindow sv
-    liftIO $ scrolledWindowSetPolicy sw PolicyAutomatic PolicyAutomatic
+    if wrapLines prefs
+        then liftIO $ scrolledWindowSetPolicy sw PolicyNever PolicyAutomatic
+        else liftIO $ scrolledWindowSetPolicy sw PolicyAutomatic PolicyAutomatic
     liftIO $ scrolledWindowSetShadowType sw ShadowIn
     modTimeRef <- liftIO $ newIORef modTime
-    let buf = IDEBuffer mbfn bn ind sv sw modTimeRef
+    let buf = IDEBuffer {
+        fileName =  mbfn,
+        bufferName = bn,
+        addedIndex = ind,
+        sourceView =sv,
+        scrolledWindow = sw,
+        modTime = modTimeRef,
+        mode = mod}
     -- events
     ids1 <- sv `afterFocusIn` makeActive buf
     ids2 <- onCompletion sv (Completion.complete sv False) Completion.cancel
@@ -484,10 +453,17 @@ builder' bs mbfn ind bn rbn ct prefs pp nb windows = do
             liftIO $ reflectIDE (do
                 case click of
                     DoubleClick -> do
-                        let isSelectChar a = (isAlphaNum a) || (a == '_')
+                        let isIdent a = isAlphaNum a || a == '\'' || a == '_'
+                        let isOp    a = isSymbol   a || a == ':'  || a == '\\' || a == '*' || a == '/' || a == '-'
+                                                     || a == '!'  || a == '@' || a == '%' || a == '&' || a == '?'
                         (startSel, endSel) <- getSelectionBounds buffer
                         mbStartChar <- getChar startSel
                         mbEndChar <- getChar endSel
+                        let isSelectChar =
+                                case mbStartChar of
+                                    Just startChar | isIdent startChar -> isIdent
+                                    Just startChar | isOp    startChar -> isOp
+                                    _                                  -> const False
                         start <- case mbStartChar of
                             Just startChar | isSelectChar startChar -> do
                                 maybeIter <- backwardFindCharC startSel (not.isSelectChar) Nothing
@@ -514,20 +490,20 @@ builder' bs mbfn ind bn rbn ct prefs pp nb windows = do
     return (Just buf,concat [ids1, ids2, ids3, ids4])
 
 
-checkModTime :: IDEBuffer -> IDEM Bool
+checkModTime :: IDEBuffer -> IDEM (Bool, Bool)
 checkModTime buf = do
     currentState' <- readIDE currentState
     case  currentState' of
-        IsShuttingDown -> return False
-        _              -> reifyIDE (\ideR -> do
+        IsShuttingDown -> return (False, False)
+        _              -> do
             let name = paneName buf
             case fileName buf of
                 Just fn -> do
-                    exists <- doesFileExist fn
+                    exists <- liftIO $ doesFileExist fn
                     if exists
                         then do
-                            nmt <- getModificationTime fn
-                            modTime' <- readIORef (modTime buf)
+                            nmt <- liftIO $ getModificationTime fn
+                            modTime' <- liftIO $ readIORef (modTime buf)
                             case modTime' of
                                 Nothing ->  error $"checkModTime: time not set " ++ show (fileName buf)
                                 Just mt ->
@@ -538,25 +514,44 @@ checkModTime buf = do
                                                   -- Praises to whoever finds out what happens and how to fix this
 #endif
                                     then do
+                                        load <- readIDE (autoLoad . prefs)
+                                        if load
+                                            then do
+                                                ideMessage Normal $ "Auto Loading " ++ fn
+                                                revert buf
+                                                return (False, True)
+                                            else do
+                                                window <- getMainWindow
+                                                resp <- liftIO $ do
                                                     md <- messageDialogNew
-                                                            Nothing []
+                                                            (Just window) []
                                                             MessageQuestion
-                                                            ButtonsYesNo
-                                                            ("File \"" ++ name ++ "\" has changed on disk. Load file from disk?")
+                                                            ButtonsNone
+                                                            ("File \"" ++ name ++ "\" has changed on disk.")
+                                                    dialogAddButton md "_Load From Disk" (ResponseUser 1)
+                                                    dialogAddButton md "_Always Load From Disk" (ResponseUser 2)
+                                                    dialogAddButton md "_Don't Load" (ResponseUser 3)
+                                                    dialogSetDefaultResponse md (ResponseUser 1)
+                                                    set md [ windowWindowPosition := WinPosCenterOnParent ]
                                                     resp <- dialogRun md
-                                                    case resp of
-                                                        ResponseYes ->  do
-                                                            reflectIDE (revert buf) ideR
-                                                            liftIO $ widgetHide md
-                                                            return False
-                                                        ResponseNo  -> do
-                                                            writeIORef (modTime buf) (Just nmt)
-                                                            widgetHide md
-                                                            return True
-                                                        _           ->  do return False
-                                                else return False
-                        else return False
-                Nothing -> return False)
+                                                    widgetDestroy md
+                                                    return resp
+                                                case resp of
+                                                    ResponseUser 1 -> do
+                                                        revert buf
+                                                        return (False, True)
+                                                    ResponseUser 2 -> do
+                                                        revert buf
+                                                        modifyIDE_ $ \ide -> ide{prefs = (prefs ide) {autoLoad = True}}
+                                                        return (False, True)
+                                                    ResponseUser 3 -> do
+                                                        nmt2 <- liftIO $ getModificationTime fn
+                                                        liftIO $ writeIORef (modTime buf) (Just nmt2)
+                                                        return (True, True)
+                                                    _           ->  do return (False, False)
+                                    else return (False, False)
+                        else return (False, False)
+                Nothing -> return (False, False)
 
 setModTime :: IDEBuffer -> IDEAction
 setModTime buf = do
@@ -573,7 +568,7 @@ fileRevert = inActiveBufContext () $ \ _ _ currentBuffer _ -> do
 
 revert :: IDEBuffer -> IDEAction
 revert buf = do
-    useCandy    <-  getCandyState
+    useCandy    <-  useCandyFor buf
     ct          <-  readIDE candy
     let name    =   paneName buf
     case fileName buf of
@@ -635,33 +630,12 @@ markLabelAsChanged nb buf = do
     modified <- getModified ebuf
     liftIO $ markLabel nb (getTopWidget buf) modified
 
-inBufContext :: alpha -> IDEBuffer -> (Notebook -> EditorBuffer -> IDEBuffer -> Int -> IDEM alpha) -> IDEM alpha
-inBufContext def ideBuf f = do
-    (pane,_)       <-  guiPropertiesFromName (paneName ideBuf)
-    nb             <-  getNotebook pane
-    mbI            <-  liftIO $notebookPageNum nb (scrolledWindow ideBuf)
-    case mbI of
-        Nothing ->  liftIO $ do
-            sysMessage Normal $ bufferName ideBuf ++ " notebook page not found: unexpected"
-            return def
-        Just i  ->  do
-            ebuf <- getBuffer (sourceView ideBuf)
-            f nb ebuf ideBuf i
-
-inActiveBufContext :: alpha -> (Notebook -> EditorBuffer -> IDEBuffer -> Int -> IDEM alpha) -> IDEM alpha
-inActiveBufContext def f = do
-    mbBuf                  <- maybeActiveBuf
-    case mbBuf of
-        Nothing         -> return def
-        Just ideBuf -> do
-            inBufContext def ideBuf f
-
 fileSaveBuffer :: Bool -> Notebook -> EditorBuffer -> IDEBuffer -> Int -> IDEM Bool
 fileSaveBuffer query nb ebuf ideBuf i = do
     ideR    <- ask
     window  <- getMainWindow
     prefs   <- readIDE prefs
-    bs      <- getCandyState
+    useCandy <- useCandyFor ideBuf
     candy   <- readIDE candy
     (panePath,connects) <- guiPropertiesFromName (paneName ideBuf)
     let mbfn = fileName ideBuf
@@ -670,14 +644,15 @@ fileSaveBuffer query nb ebuf ideBuf i = do
         Nothing     -> throwIDE "fileSave: Page not found"
         Just page   ->
             if isJust mbfn && query == False
-                then do modifiedOnDisk <- checkModTime ideBuf -- The user is given option to reload
+                then do (modifiedOnDiskNotLoaded, modifiedOnDisk) <- checkModTime ideBuf -- The user is given option to reload
                         modifiedInBuffer <- getModified ebuf
-                        if (modifiedOnDisk || modifiedInBuffer)
+                        if (modifiedOnDiskNotLoaded || modifiedInBuffer)
                             then do
-                                fileSave' (forceLineEnds prefs) (removeTBlanks prefs) nb ideBuf bs candy $fromJust mbfn
+                                fileSave' (forceLineEnds prefs) (removeTBlanks prefs) nb ideBuf
+                                    useCandy candy $fromJust mbfn
                                 setModTime ideBuf
                                 return True
-                            else return False
+                            else return modifiedOnDisk
                 else reifyIDE $ \ideR   ->  do
                     dialog <- fileChooserDialogNew
                                     (Just $ "Save File")
@@ -687,6 +662,9 @@ fileSaveBuffer query nb ebuf ideBuf i = do
                                 ,ResponseCancel)  --you can use stock buttons
                                 ,("gtk-save"
                                 , ResponseAccept)]
+                    case mbfn of
+                        Just fn -> fileChooserSelectFilename dialog fn >> return ()
+                        Nothing -> return ()
                     widgetShow dialog
                     response <- dialogRun dialog
                     mbFileName <- case response of
@@ -702,29 +680,31 @@ fileSaveBuffer query nb ebuf ideBuf i = do
                             resp <- if dfe
                                 then do md <- messageDialogNew (Just window) []
                                                 MessageQuestion
-                                                ButtonsYesNo
-                                                "File already exist. Overwrite?"
+                                                ButtonsCancel
+                                                "File already exist."
+                                        dialogAddButton md "_Overwrite" ResponseYes
+                                        dialogSetDefaultResponse md ResponseCancel
+                                        set md [ windowWindowPosition := WinPosCenterOnParent ]
                                         resp <- dialogRun md
                                         widgetHide md
                                         return resp
                                 else return ResponseYes
                             case resp of
                                 ResponseYes -> do
-                                    cfn <- canonicalizePath fn
-
                                     reflectIDE (do
-                                        fileSave' (forceLineEnds prefs) (removeTBlanks prefs) nb ideBuf bs candy cfn
+                                        fileSave' (forceLineEnds prefs) (removeTBlanks prefs)
+                                            nb ideBuf useCandy candy fn
                                         closePane ideBuf
-                                        newTextBuffer panePath (takeFileName fn) (Just cfn)
-                                        )   ideR
+                                        cfn <- liftIO $ myCanonicalizePath fn
+                                        newTextBuffer panePath (takeFileName cfn) (Just cfn)
+                                        ) ideR
                                     return True
-                                ResponseNo -> return False
                                 _          -> return False
     where
         fileSave' :: Bool -> Bool -> Notebook -> IDEBuffer -> Bool -> CandyTable -> FilePath -> IDEAction
-        fileSave' forceLineEnds removeTBlanks nb ideBuf bs ct fn = do
+        fileSave' forceLineEnds removeTBlanks nb ideBuf useCandy candyTable fn = do
             buf     <-   getBuffer $ sourceView ideBuf
-            text    <-   getCandylessText ct buf
+            text    <-   getCandylessText candyTable buf
             let text' = if removeTBlanks
                             then unlines $ map removeTrailingBlanks $lines text
                             else text
@@ -751,7 +731,7 @@ fileCheckBuffer :: Notebook -> EditorBuffer -> IDEBuffer -> Int -> IDEM Bool
 fileCheckBuffer nb ebuf ideBuf i = do
     let mbfn = fileName ideBuf
     if isJust mbfn
-        then do modifiedOnDisk      <- checkModTime ideBuf -- The user is given option to reload
+        then do (_, modifiedOnDisk) <- checkModTime ideBuf -- The user is given option to reload
                 modifiedInBuffer    <- getModified ebuf
                 return (modifiedOnDisk || modifiedInBuffer)
         else return False
@@ -788,13 +768,13 @@ fileClose' nb ebuf currentBuffer i = do
             then do
                 md <- messageDialogNew (Just window) []
                                             MessageQuestion
-                                            ButtonsNone
+                                            ButtonsCancel
                                             ("Save changes to document: "
                                                 ++ paneName currentBuffer
                                                 ++ "?")
                 dialogAddButton md "_Save" ResponseYes
                 dialogAddButton md "_Don't Save" ResponseNo
-                dialogAddButton md "_Cancel" ResponseCancel
+                set md [ windowWindowPosition := WinPosCenterOnParent ]
                 resp <- dialogRun md
                 widgetDestroy md
                 case resp of
@@ -875,6 +855,7 @@ fileOpen :: IDEAction
 fileOpen = do
     window <- getMainWindow
     prefs <- readIDE prefs
+    mbBuf <- maybeActiveBuf
     mbFileName <- liftIO $ do
         dialog <- fileChooserDialogNew
                         (Just $ "Open File")
@@ -884,6 +865,9 @@ fileOpen = do
                     ,ResponseCancel)
                     ,("gtk-open"
                     ,ResponseAccept)]
+        case mbBuf >>= fileName of
+            Just fn -> fileChooserSetCurrentFolder dialog (dropFileName $ fn) >> return ()
+            Nothing -> return ()
         widgetShow dialog
         response <- dialogRun dialog
         case response of
@@ -906,24 +890,30 @@ fileOpen = do
 fileOpenThis :: FilePath -> IDEAction
 fileOpenThis fp =  do
     prefs <- readIDE prefs
-    fpc <-  liftIO $canonicalizePath fp
+    fpc <-  liftIO $ myCanonicalizePath fp
     buffers <- allBuffers
     let buf = filter (\b -> case fileName b of
                         Just fn -> equalFilePath fn fpc
                         Nothing -> False) buffers
     case buf of
         hdb:tl -> do
-            md <- liftIO $messageDialogNew
-                    Nothing []
-                    MessageQuestion
-                    ButtonsYesNo
-                    ("Buffer already open. " ++
-                     "Make active instead of opening a second time?")
-            resp <- liftIO $dialogRun md
-            liftIO $ widgetDestroy md
+            window <- getMainWindow
+            resp <- liftIO $ do
+                md <- messageDialogNew
+                        (Just window) []
+                        MessageQuestion
+                        ButtonsNone
+                        "Buffer already open."
+                dialogAddButton md "Make _Active" (ResponseUser 1)
+                dialogAddButton md "_Open Second" (ResponseUser 2)
+                dialogSetDefaultResponse md (ResponseUser 1)
+                set md [ windowWindowPosition := WinPosCenterOnParent ]
+                resp <- dialogRun md
+                widgetDestroy md
+                return resp
             case resp of
-                ResponseNo  ->  reallyOpen prefs fpc
-                _           ->  makeActive hdb
+                ResponseUser 2 -> reallyOpen prefs fpc
+                _              -> makeActive hdb
         [] -> reallyOpen prefs fpc
     where
         reallyOpen prefs fpc =   do
@@ -954,62 +944,21 @@ editSelectAll = inActiveBufContext () $ \_ ebuf _ _ -> do
 
 editCut :: IDEAction
 editCut = inActiveBufContext () $ \_ ebuf _ _ -> do
-    cb   <- liftIO $ atomNew "GDK_SELECTION_CLIPBOARD"
-    clip <- liftIO $ clipboardGet cb
+    clip <- liftIO $ clipboardGet selectionClipboard
     cutClipboard ebuf clip True
 
 editCopy :: IDEAction
 editCopy = inActiveBufContext () $ \_ ebuf _ _ -> do
-    cb   <- liftIO $ atomNew "GDK_SELECTION_CLIPBOARD"
-    clip <- liftIO $ clipboardGet cb
+    clip <- liftIO $ clipboardGet selectionClipboard
     copyClipboard ebuf clip
 
 editPaste :: IDEAction
 editPaste = inActiveBufContext () $ \_ ebuf _ _ -> do
-    cb   <- liftIO $ atomNew "GDK_SELECTION_CLIPBOARD"
     mark <- getInsertMark ebuf
     iter <- getIterAtMark ebuf mark
-    clip <- liftIO $ clipboardGet cb
+    clip <- liftIO $ clipboardGet selectionClipboard
     pasteClipboard ebuf clip iter True
 
-getStartAndEndLineOfSelection :: EditorBuffer -> IDEM (Int,Int)
-getStartAndEndLineOfSelection ebuf = do
-    startMark   <- getInsertMark ebuf
-    endMark     <- getSelectionBoundMark ebuf
-    startIter   <- getIterAtMark ebuf startMark
-    endIter     <- getIterAtMark ebuf endMark
-    startLine   <- getLine startIter
-    endLine     <- getLine endIter
-    let (startLine',endLine',endIter') = if endLine >=  startLine
-            then (startLine,endLine,endIter)
-            else (endLine,startLine,startIter)
-    b           <- startsLine endIter'
-    let endLineReal = if b && endLine /= startLine then endLine' - 1 else endLine'
-    return (startLine',endLineReal)
-
-doForSelectedLines :: [a] -> (EditorBuffer -> Int -> IDEM a) -> IDEM [a]
-doForSelectedLines d f = inActiveBufContext d $ \_ ebuf currentBuffer _ -> do
-    (start,end) <- getStartAndEndLineOfSelection ebuf
-    mapM (f ebuf) [start .. end]
-
-editComment :: IDEAction
-editComment = do
-    doForSelectedLines [] $ \ebuf lineNr -> do
-        sol <- getIterAtLine ebuf lineNr
-        insert ebuf sol "--"
-    return ()
-
-editUncomment :: IDEAction
-editUncomment = do
-    doForSelectedLines [] $ \ebuf lineNr -> do
-        sol <- getIterAtLine ebuf lineNr
-        sol2 <- forwardCharsC sol 2
-        str   <- getText ebuf sol sol2 True
-        if str == "--"
-            then do delete ebuf sol sol2
-            else return ()
-    return ()
-
 editShiftLeft :: IDEAction
 editShiftLeft = do
     prefs <- readIDE prefs
@@ -1042,34 +991,6 @@ editShiftRight = do
         insert ebuf sol str
     return ()
 
-editToCandy :: IDEAction
-editToCandy = do
-    ct <- readIDE candy
-    inActiveBufContext () $ \_ ebuf _ _ -> do
-        transformToCandy ct ebuf
-
-editFromCandy :: IDEAction
-editFromCandy = do
-    ct      <-  readIDE candy
-    inActiveBufContext () $ \_ ebuf _ _ -> do
-        transformFromCandy ct ebuf
-
-editKeystrokeCandy :: Maybe Char -> IDEAction
-editKeystrokeCandy c = do
-    ct <- readIDE candy
-    inActiveBufContext () $ \_ ebuf _ _ -> do
-        keystrokeCandy ct c ebuf
-
-editCandy :: IDEAction
-editCandy = do
-    ct      <- readIDE candy
-    buffers <- allBuffers
-    gtkbufs <- mapM (\ b -> getBuffer (sourceView b)) buffers
-    bs <- getCandyState
-    if bs
-        then mapM_ (transformToCandy ct) gtkbufs
-        else mapM_ (transformFromCandy ct) gtkbufs
-
 alignChar :: Char -> IDEAction
 alignChar char = do
     positions     <- positionsOfChar
@@ -1153,9 +1074,9 @@ selectedTextOrCurrentLine = do
 
 selectedLocation :: IDEM (Maybe (Int, Int))
 selectedLocation = do
-    useCandy    <- getCandyState
     candy'      <- readIDE candy
     inActiveBufContext Nothing $ \_ ebuf currentBuffer _ -> do
+        useCandy   <- useCandyFor currentBuffer
         (start, _) <- getSelectionBounds ebuf
         line       <- getLine start
         lineOffset <- getLineOffset start
@@ -1164,13 +1085,11 @@ selectedLocation = do
             else return (line, lineOffset)
         return $ Just res
 
--- " ++ ++ ++ alpha
-
 insertTextAfterSelection :: String -> IDEAction
 insertTextAfterSelection str = do
-        candy'       <- readIDE candy
-        useCandy     <- getCandyState
-        inActiveBufContext () $ \_ ebuf currentBuffer _ -> do
+    candy'       <- readIDE candy
+    inActiveBufContext () $ \_ ebuf currentBuffer _ -> do
+        useCandy     <- useCandyFor currentBuffer
         hasSelection <- hasSelection ebuf
         when hasSelection $ do
             realString <-  if useCandy then stringToCandy candy' str else return str
@@ -1181,18 +1100,11 @@ insertTextAfterSelection str = do
             i2         <- forwardCharsC i1 (length str)
             selectRange ebuf i1 i2
 
-selectedModuleName :: IDEM (Maybe String)
-selectedModuleName = do
-    candy' <- readIDE candy
-    inActiveBufContext Nothing $ \_ ebuf currentBuffer _ -> do
-        case fileName currentBuffer of
-            Just filePath -> liftIO $ moduleNameFromFilePath filePath
-            Nothing       -> return Nothing
-
 -- | Returns the package, to which this buffer belongs, if possible
 belongsToPackage :: IDEBuffer -> IDEM(Maybe IDEPackage)
-belongsToPackage ideBuf | fileName ideBuf == Nothing = return Nothing
-                        | otherwise                 = do
+belongsToPackage ideBuf | fileName ideBuf == Nothing        = return Nothing
+                        | not (isHaskellMode (mode ideBuf))  = return Nothing
+                        | otherwise                        = do
     bufferToProject' <-  readIDE bufferProjCache
     ws               <-  readIDE workspace
     let fp           =   fromJust (fileName ideBuf)
@@ -1227,3 +1139,15 @@ belongsToPackage' fp mbModuleName Nothing pack =
         else Nothing
 
 belongsToWorkspace b =  belongsToPackage b >>= return . isJust
+
+useCandyFor :: IDEBuffer -> IDEM Bool
+useCandyFor aBuffer = do
+    use <- getCandyState
+    return (use && isHaskellMode (mode aBuffer))
+
+editCandy = do
+    use <- getCandyState
+    buffers <- allBuffers
+    if use
+        then mapM_ (modeEditToCandy . mode) buffers
+        else mapM_ (modeEditFromCandy . mode) buffers
