@@ -43,7 +43,6 @@ import IDE.TextEditor
 import IDE.Pane.Modules
 import IDE.Pane.SourceBuffer
 import IDE.Pane.Info
-import IDE.Pane.References
 import IDE.Pane.Log
 import IDE.Pane.Preferences
 import IDE.Pane.PackageFlags
@@ -59,6 +58,7 @@ import IDE.Pane.Errors (ErrorsState(..))
 import Control.Exception (SomeException(..))
 import IDE.Pane.Workspace (WorkspaceState(..))
 import IDE.Workspaces (workspaceOpenThis)
+import IDE.Completion (setCompletionSize)
 
 
 -- ---------------------------------------------------------------------
@@ -75,7 +75,6 @@ data PaneState      =   BufferSt BufferState
                     |   LogSt LogState
                     |   InfoSt InfoState
                     |   ModulesSt ModulesState
-                    |   ReferencesSt ReferencesState
                     |   PrefsSt PrefsState
                     |   FlagsSt FlagsState
                     |   SearchSt SearchState
@@ -92,7 +91,6 @@ asPaneState s | isJust ((cast s) :: Maybe BufferState)      =   BufferSt (fromJu
 asPaneState s | isJust ((cast s) :: Maybe LogState)         =   LogSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe InfoState)        =   InfoSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe ModulesState)     =   ModulesSt (fromJust $ cast s)
-asPaneState s | isJust ((cast s) :: Maybe ReferencesState)  =   ReferencesSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe PrefsState)       =   PrefsSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe FlagsState)       =   FlagsSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe SearchState)      =   SearchSt (fromJust $ cast s)
@@ -109,7 +107,6 @@ recover pp (BufferSt p)         =   recoverState pp p >> return ()
 recover pp (LogSt p)            =   recoverState pp p >> return ()
 recover pp (InfoSt p)           =   recoverState pp p >> return ()
 recover pp (ModulesSt p)        =   recoverState pp p >> return ()
-recover pp (ReferencesSt p)     =   recoverState pp p >> return ()
 recover pp (PrefsSt p)          =   recoverState pp p >> return ()
 recover pp (FlagsSt p)          =   recoverState pp p >> return ()
 recover pp (SearchSt p)         =   recoverState pp p >> return ()
@@ -142,6 +139,7 @@ data SessionState = SessionState {
     ,   layoutS             ::   PaneLayout
     ,   population          ::   [(Maybe PaneState,PanePath)]
     ,   windowSize          ::   (Int,Int)
+    ,   completionSize      ::   (Int,Int)
     ,   workspacePath       ::   Maybe FilePath
     ,   activePaneN         ::   Maybe String
     ,   toolbarVisibleS     ::   Bool
@@ -156,6 +154,7 @@ defaultSession = SessionState {
     ,   layoutS             =   TerminalP Map.empty (Just TopP) (-1) Nothing Nothing
     ,   population          =   []
     ,   windowSize          =   (1024,768)
+    ,   completionSize      =   (750,400)
     ,   workspacePath       =   Nothing
     ,   activePaneN         =   Nothing
     ,   toolbarVisibleS     =   True
@@ -205,6 +204,12 @@ sessionDescr = [
             (pairParser intParser)
             windowSize
             (\(c,d) a -> a{windowSize = (c,d)})
+    ,   mkFieldS
+            (paraName <<<- ParaName "Completion size" $ emptyParams)
+            (PP.text . show)
+            (pairParser intParser)
+            completionSize
+            (\(c,d) a -> a{completionSize = (c,d)})
     ,   mkFieldS
             (paraName <<<- ParaName "Workspace" $ emptyParams)
             (PP.text . show)
@@ -273,6 +278,7 @@ saveSessionAs sessionPath mbSecondPath = do
             layout          <-  mkLayout
             population      <-  getPopulation
             size            <-  liftIO $ windowGetSize wdw
+            (completionSize,_) <- readIDE completion
             mbWs            <-  readIDE workspace
             activePane'     <-  getActivePane
             let activeP =   case activePane' of
@@ -290,6 +296,7 @@ saveSessionAs sessionPath mbSecondPath = do
             ,   layoutS             =   layout
             ,   population          =   population
             ,   windowSize          =   size
+            ,   completionSize      =   completionSize
             ,   workspacePath       =   case mbWs of
                                             Nothing -> Nothing
                                             Just ws -> Just (wsFile ws)
@@ -467,6 +474,7 @@ recoverSession sessionPath = catchIDE (do
         if (fst . findbarState) sessionSt
             then showFindbar
             else hideFindbar
+        setCompletionSize (completionSize sessionSt)
         modifyIDE_ (\ide -> ide{recentFiles = recentOpenedFiles sessionSt,
                                         recentWorkspaces = recentOpenedWorksp sessionSt})
         return (toolbarVisibleS sessionSt, (fst . findbarState) sessionSt))
